/**************************************************************************
 * ESPCONF.C                                                              *
 * Configure ESP enhanced mode receive and transmit                       *
 **************************************************************************/

/*** NOTICE *** NOTICE *** NOTICE *** NOTICE *** NOTICE *** NOTICE ***
 ***                                                               ***
DISCLAIMER: THE PRODUCT IS PROVIDED TO DEVELOPER "AS IS" WITHOUT WARRANTY
OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE 
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE,
AND THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PRODUCT IS 
ASSUMED BY DEVELOPER, INCLUDING ALL COSTS OF SERVICING, REPAIRS OR CORRECTIONS.  
DEVELOPER AGREES THAT HAYES WILL NOT BE LIABLE FOR ANY DAMAGES DIRECTLY OR 
INDIRECTLY RELATED TO THE USE OF THE PRODUCT, INCLUDING BUT NOT LIMITED
TO, LOST PROFITS, INDIRECT, SPECIAL OR CONSEQUENTIAL DAMAGES, WHETHER BASED
ON CONTRACT, TORT OR ANY OTHER LEGAL THEORY.  DEVELOPER ACKNOWLEDGES THAT 
THE PRODUCT IS STILL UNDER DEVELOPMENT AND THAT HAYES DOES NOT WARRANT 
THAT THE PRODUCT IN ITS PRESENT OR FUTURE FORM WILL MEET DEVELOPER'S NEEDS.
 ***                                                               ***
 *** NOTICE *** NOTICE *** NOTICE *** NOTICE *** NOTICE *** NOTICE ***/

/***********************************************************************/
/*    Copyright 1993 Hayes Microcomputer Products, Inc.                */
/*    All rights reserved.                                             */
/*    Use of copyright notice does not imply publication.              */
/***********************************************************************/
/*                                                                     */
/*                                                                     */
/*     NOTE:   This code is specifically for the ESP2 board.           */
/*                                                                     */
/*                     T E S T    C O D E                              */
/*                                                                     */
/*     This code uses the Hayes Enhanced Serial Interface (ESI) Ver 2  */    
/*     to operate the ESP2 adapter.  Please refer to that manual for   */
/*     more detail on the commands.                                    */
/***********************************************************************/

#include <stdio.h>
#include <conio.h>
#include <memory.h>
#include <stdlib.h>
#include <string.h>
#include <dos.h>
#include "esptype.h"
#include "espfunc.h"


/***********************************************************************/
/*     DoEspConfig                                                     */
/*     This is the routine that starts all configuration of the ESP    */
/*     port.  Most of the configuration data is read in from ESPTEST.  */
/*     CFG.                                                            */
/***********************************************************************/


void   DoEspConfig(void)
{

   ESPCMDBLOCK cmdBlock;
   int i;
   int NumOfPorts = 0;

/***********************************************************************/
/* Initialize variables                                                */
/***********************************************************************/
   InitVariables();

/***********************************************************************/
/* We support up two ports in our test, however only one port can      */
/* operate at any given time.                                          */
/***********************************************************************/
   if( P38Ports[0].PortState)
       NumOfPorts++;
   if( P38Ports[1].PortState)
       NumOfPorts++;

/***********************************************************************/
/* We go through a loop to configure all ports.                        */
/***********************************************************************/

   for (i = 0; i < NumOfPorts; i++)
   {

/***********************************************************************/
/*  Do a reset to clean up old states                                  */
/***********************************************************************/
       if (computer == AT)
       {
           cmdBlock.ioBase = P38Ports[i].EnhancedAddress;
           cmdBlock.command = ESP_CMD_RESET;                             
           IssueEspCmd( &cmdBlock );
       }

/***********************************************************************/
/* The dma timeout is read in from the config file.                    */
/* It will be no greater than 255.                                     */
/***********************************************************************/
       if (((P38Ports[i].RxMode == DMA_RX) ||
            (P38Ports[i].TxMode == DMA_TX)))
       { 
           cmdBlock.command = ESP_CMD_SETDMATIME; 
           cmdBlock.writeParms[0] = P38Ports[i].dmaTime;
           IssueEspCmd( &cmdBlock );
       }

/***********************************************************************/
/* Mask out the service mask while we do our configuration work.       */
/* This will prevent any unwanted interrupts.                          */
/***********************************************************************/
       cmdBlock.command = ESP_CMD_SETSVCMASK;
       cmdBlock.writeParms[0] = 0x00;     
       IssueEspCmd( &cmdBlock );

/***********************************************************************/
/* This routine sets up the Interrupt service routine and the PIC for  */
/* our ISR.                                                            */
/***********************************************************************/
       SetUpHardwareHandler((BYTE) i);   

/***********************************************************************/
/*  Set Enhanced mode interrupt and DMA channel if applicable.         */
/*  Both the MCA and AT buses need to have bit 0 set to enable         */
/*  enhanced IRQ but the dma channel and IRQ values are set for AT     */
/*  bus only because the POS registers are setup for MCA and have      */
/*  already handled the DMA level and IRQ.                             */
/***********************************************************************/
       cmdBlock.command = ESP_CMD_SETIRQ; 
       cmdBlock.writeParms[0] = 1;
       if (computer == AT)
       {
           cmdBlock.writeParms[0] = P38Ports[i].IRQValues->DmaIrq;
           if ((P38Ports[i].TxMode == DMA_TX)  ||  
               (P38Ports[i].RxMode == DMA_RX))
           {
               if (P38Ports[i].dmaChannel == 1)
                   cmdBlock.writeParms[0] |= 0x10;
               else
                   cmdBlock.writeParms[0] |= 0x30;
           }
       }
       IssueEspCmd( &cmdBlock );

/***********************************************************************/
/* If P-38 we issue the ENHANCED IRQ ESP command (1Fh) so that we can  */
/* support the additional IRQ's available for P-38.                    */
/***********************************************************************/
       if (P38Ports[i].IRQValues->ESPIrq)
       {
           cmdBlock.command = ESP_CMD_ENHANCED_IRQ; 
           cmdBlock.writeParms[0] = P38Ports[i].IRQValues->ESPIrq;
           IssueEspCmd( &cmdBlock );
       }
/***********************************************************************/
/*     This call does the rest of the initialization work.             */
/***********************************************************************/

       InitializePort (i);


   }


   return;
}


/***********************************************************************/
/*     InitializePort                                                  */
/*  Initialize the port's table structure entry parameters.            */
/***********************************************************************/

void InitializePort(WORD Port )
{

   ESPCMDBLOCK cmdBlock;


/***********************************************************************/
/*  Set the error mask for errors that we want to be interrupted for.  */
/***********************************************************************/
   cmdBlock.ioBase = P38Ports[Port].EnhancedAddress;
   cmdBlock.command = ESP_CMD_SETERRMASK;
   cmdBlock.writeParms[0] = ESP_ERR1_RXTO | ESP_ERR1_RXOF | ESP_ERR1_PARITY;
   cmdBlock.writeParms[0] |= ESP_ERR1_FRAME | ESP_ERR1_BRK | ESP_ERR1_MSR;
   cmdBlock.writeParms[1] = (ESP_ERR2_OFFREMOTE | ESP_ERR2_OFFLOCAL | ESP_ERR2_ONLOCAL);
   IssueEspCmd( &cmdBlock );

/***********************************************************************/
/*     Flush our receive queue.                                        */
/***********************************************************************/
   cmdBlock.command = ESP_CMD_FLUSHRX;     
   IssueEspCmd( &cmdBlock );

/***********************************************************************/
/*     Flush our transmit queue.                                       */
/***********************************************************************/
   cmdBlock.command = ESP_CMD_FLUSHTX;     
   IssueEspCmd( &cmdBlock );

/***********************************************************************/
/* Set our mode for enhanced and for the transfer mode selected in the */
/* ESPTEST.CFG configuration file.                                     */
/*     We support DMA, Programmed IO and Memory map.                   */
/***********************************************************************/

   cmdBlock.writeParms[0] = ENHANCED;      // turn on enhanced mode
//   cmdBlock.writeParms[0] = 02;      // turn on enhanced mode
//   cmdBlock.writeParms[0] = 82;      // turn on enhanced mode
   cmdBlock.writeParms[0] |= (P38Ports[Port].RxMode|P38Ports[Port].TxMode);     
   cmdBlock.command = ESP_CMD_SETMODE;     
   IssueEspCmd( &cmdBlock );

/***********************************************************************/
/* Flow on our local transmitter.                                      */
/***********************************************************************/
   cmdBlock.command = ESP_CMD_XONLOCAL;     
   IssueEspCmd( &cmdBlock );

   PortDataConfigure(Port );

   return;
}


/* ============================== INITVARIABLES ==============================
 * Initialize global variables
 * Enter:  none
 * Return: some global variables initialized
 */

void   InitVariables(void)
{
   WORD    i;


/***********************************************************************/
/* Clear all counter fields.                                           */
/***********************************************************************/
   P38Ports[0].transmitBytes = P38Ports[1].transmitBytes =
   P38Ports[0].receiveBytes = P38Ports[1].receiveBytes = 0l;
   P38Ports[0].NumberofTransfers = P38Ports[1].NumberofTransfers = 0;
   P38Ports[0].parityErrors = P38Ports[1].parityErrors =
   P38Ports[0].framingErrors = P38Ports[1].framingErrors =
   P38Ports[0].compareErrors = P38Ports[1].compareErrors = 0l;
   P38Ports[0].ISREntry = P38Ports[1].ISREntry = 0l;
   P38Ports[0].ReceiveToutCtr = P38Ports[1].ReceiveToutCtr = 0l;
   P38Ports[0].ModemStatusRegister = P38Ports[0].ModemStatusRegister = 0l;



/***********************************************************************/
/*  Set up Receive buffer initial values.....                          */
/***********************************************************************/
   P38Ports[0].rcvbfrStart = &rx_buffer[0] [0];
   P38Ports[1].rcvbfrStart = &rx_buffer[1] [0];
   P38Ports[0].rcvbfrEnd = (P38Ports[0].rcvbfrStart + 
                           ((P38Ports[0].rcvbfrSize  + 1) - 1));
   P38Ports[1].rcvbfrEnd = (P38Ports[1].rcvbfrStart + 
                           ((P38Ports[1].rcvbfrSize  + 1) - 1));
   P38Ports[0].rcvbfrDrvr     = P38Ports[0].rcvbfrStart;
   P38Ports[0].rcvbfrPC       = P38Ports[0].rcvbfrStart;
   P38Ports[1].rcvbfrDrvr     = P38Ports[1].rcvbfrStart;
   P38Ports[1].rcvbfrPC       = P38Ports[1].rcvbfrStart;


   P38Ports[0].xmtbfrSize = P38Ports[1].xmtbfrSize = (MAX_COUNT/2);
   for (i = 0; i < (MAX_COUNT/2); i++)
   {
       (BYTE) tx_buffer[0][i] = (BYTE)save_tx_buffer[i];
   } 



}



/* ============================ GET_PORT_CONFIG =============================
 * Gets program configuration data for one port from ESPTEST.CFG
 * ENTER:       Port = 1 or 2
 * RETURN:      0=OK, 1=file not found, 2=file format error
 *             See ESPTEST.CFG file for configuration parameters and their
 *             definitions.
 */
int   get_port_config( int port )
{
   char    string[81];
   int     temp;
   LONG    Baud;
   BYTE    i,j;
   WORD    EnhAddr;
   
   /************************/
   /*  Option number 1     */
   /*  Activate Port Y/N   */
   /************************/


   sprintf (ConfigFileStr, "%s%s", PortConfig[1].String,
               PortConfig[1].Format);
   result = fscanf( file, ConfigFileStr, string );

   if(result != 1)  
   {
       sprintf (ConfigFileStr, "-> %.60s", PortConfig[1].String);
       return( 3 );
   }
   if( strcmpi( string, "n" ) == 0 ) 
   {
       P38Ports[port-1].PortState = 0;
       return(0);
   }
   else if( strcmpi( string, "y" ) == 0 ) 
        {
           P38Ports[port-1].PortState = 1;
        }
        else
        {
           sprintf (ConfigFileStr, "-> %.50s %.20s",
                      PortConfig[1].String, string);
           return (2);
        }

   /*******************************/
   /*     Option  number 2        */
   /*     Enhanced address        */  
   /*******************************/

   sprintf (ConfigFileStr, "%s%s", PortConfig[2].String,
               PortConfig[2].Format);
   result = fscanf( file, ConfigFileStr, &temp );
   if(result != 1)  
   {
       sprintf (ConfigFileStr,"-> %.60s", PortConfig[2].String);
       return( 3 );
   }

   if (computer == AT)
   {
       for (i = 0; i < MAX_AT_ENH_ADDRESS_INDEX; i++)
       {
           EnhAddr = EnhancedAddressAT[i];    
           for (j = 0; j < 8; j++)
           {    
               if ((WORD) temp == (WORD) EnhAddr)
               {
                   P38Ports[port-1].EnhancedAddress = EnhAddr;
                   break;
               }
               EnhAddr += 8;
           }
           if (P38Ports[port-1].EnhancedAddress == EnhAddr)
               break;
       }
   }           
   else
   {
       for (i = 0; i < MAX_MCA_ENH_ADDRESS_INDEX; i++)
       {
           EnhAddr = EnhancedAddressMCA[i];    
           for (j = 0; j < 4; j++)
           {    
               if ((WORD) temp == (WORD) EnhAddr)
               {
                   P38Ports[port-1].EnhancedAddress = EnhAddr;
                   break;
               }
               EnhAddr += 8;
           }
           if (P38Ports[port-1].EnhancedAddress == EnhAddr)
               break;
       }
   }           


   if (!P38Ports[port-1].EnhancedAddress)
   {
       sprintf (ConfigFileStr,"-> %.60s", PortConfig[2].String);
       return( 3 );
   }

/***********************************************************************/
/*  Look for the ESP adapters now.                                     */
/*   This routine will look for the ESP2 signature.                    */
/***********************************************************************/
   if (! FindESPPorts(port-1) )
   {
       if ((port-1) != 0)
       return (4);
   }


   /*************************/
   /*  Option number 3      */
   /*  FlowControl Byte 1   */
   /*************************/

   sprintf (ConfigFileStr, "%s%s", PortConfig[3].String,
               PortConfig[3].Format);
   result = fscanf( file, ConfigFileStr, &temp);
   if(result != 1)  
   {
       sprintf (ConfigFileStr, "-> %.60s", PortConfig[3].String);
       return( 3 );
   }

   if ( temp > 0xff)
   {
       sprintf (ConfigFileStr, "-> %.60s",
               PortConfig[3].String);
       return (2);
   }

   P38Ports[port-1].flowCtrlEsp1 = (BYTE) temp;

   /**************************/
   /*  Option number 4       */
   /*  FlowControl Byte 2    */
   /**************************/

   sprintf (ConfigFileStr, "%s%s", PortConfig[4].String,
               PortConfig[4].Format);
   result = fscanf( file, ConfigFileStr, &temp);
   if(result != 1)  
   {
       sprintf (ConfigFileStr, "-> %.60s", PortConfig[4].String);
       return( 3 );
   }

   if ( temp > 0xff)
   {
       sprintf (ConfigFileStr, "-> %.60s",
               PortConfig[4].String);
       return (2);
   }

   P38Ports[port-1].flowCtrlEsp2 = (BYTE) temp;

   /************************/
   /*  Option number 5     */
   /*  RX Transfer type    */
   /************************/
/***********************************************************************/
/*     Receive transfer mode supported:                                */
/*                 Programmed IO                                       */
/*                 Memory Mapped                                       */
/*                 DMA                                                 */
/***********************************************************************/
   sprintf (ConfigFileStr, "%s%s", PortConfig[5].String,
               PortConfig[5].Format);
   result = fscanf( file, ConfigFileStr, string );
   if(result != 1)  
   {
       sprintf (ConfigFileStr,"-> %.60s", PortConfig[5].String);
       return( 3 );
   }
   if( strcmpi( string, "io" ) == 0 ) 
       P38Ports[port-1].RxMode = CHAR_RX;
   else if( strcmpi( string, "dma" ) == 0 ) 
           P38Ports[port-1].RxMode = DMA_RX;
        else if( strcmpi( string, "memmap" ) == 0 ) 
               P38Ports[port-1].RxMode = MEMORY_MAPPED;
             else
             {
                   sprintf (ConfigFileStr, "-> %.50s %.20s",
                            PortConfig[5].String, string);
                   return (2);
             }
   /************************/
   /*  Option number 6     */
   /*  TX transfer type    */
   /************************/
/***********************************************************************/
/*     Transmit transfer mode supported:                               */
/*                 Programmed IO                                       */
/*                 Memory Mapped                                       */
/*                 DMA                                                 */
/***********************************************************************/

   sprintf (ConfigFileStr, "%s%s", PortConfig[6].String,
               PortConfig[6].Format);
   result = fscanf( file, ConfigFileStr, string );
   if(result != 1)  
   {
       sprintf (ConfigFileStr,"-> %.60s", PortConfig[6].String);
       return( 3 );
   }
   if( strcmpi( string, "io" ) == 0 ) 
       P38Ports[port-1].TxMode = CHAR_TX;
   else if( strcmpi( string, "dma" ) == 0 ) 
           P38Ports[port-1].TxMode = DMA_TX;
        else if( strcmpi( string, "memmap" ) == 0 ) 
               P38Ports[port-1].TxMode = MEMORY_MAPPED;
             else
             {
                   sprintf (ConfigFileStr, "-> %.50s %.20s",
                            PortConfig[6].String, string);
                   return (2);
             }


   /**************************/
   /*  Option number 7       */
   /*  Transfer size         */
   /*  no longer configurable*/
   /**************************/
   P38Ports[port-1].rcvbfrSize = MAX_RECV_COUNT;
   P38Ports[port-1].xmtbfrSize = (MAX_COUNT/2);


   /************************/
   /*  Option number 8     */
   /*  Baud                */
   /************************/

   sprintf (ConfigFileStr, "%s%s", PortConfig[8].String,
               PortConfig[8].Format);
   result = fscanf( file, ConfigFileStr, &Baud );
   if(result != 1)  
   {
       sprintf (ConfigFileStr,"-> %.60s", PortConfig[8].String);
       return( 3 );
   }
   for (i = 0; i < MAX_BIT_RATE_INDEX + 1; i++)
   {
       if (BaudRateTable[i] ==  Baud)
       {
           P38Ports[port-1].bitRate = i;
           break;
       }
       if  (i == MAX_BIT_RATE_INDEX)  
       {
           sprintf (ConfigFileStr,"-> %.60s ", PortConfig[8].String);
           return( 3 );
       }

   }

   /************************/
   /*  Option number 9     */
   /*  Data bits           */
   /************************/

   sprintf (ConfigFileStr, "%s%s", PortConfig[9].String,
               PortConfig[9].Format);
   result = fscanf( file, ConfigFileStr, &temp );
   if(result != 1)  
   {
       sprintf (ConfigFileStr,"-> %.60s", PortConfig[9].String);
       return( 3 );
   }
   for (i = 0; i < MAX_DATA_BITS_INDEX + 1; i++)
   {

       if (DataBitsTable[i].Value == (BYTE) temp)
       {
           P38Ports[port-1].dataBits = i;
           break;
       }
       if (i == MAX_DATA_BITS_INDEX)
       {
           sprintf (ConfigFileStr,"-> %.60s ", PortConfig[9].String);
           return( 3 );
       }

   }
   
   /************************/
   /*  Option number 10    */
   /*    parity            */
   /************************/

   sprintf (ConfigFileStr, "%s%s", PortConfig[10].String,
               PortConfig[10].Format);
   result = fscanf( file, ConfigFileStr, string );
   if(result != 1)  
   {
       sprintf (ConfigFileStr,"-> %.60s", PortConfig[10].String);
       return( 3 );
   }

   for (i = 0; i < MAX_PARITY_INDEX + 1; i++)
   {
       if( strcmpi( string, ParityTable[i].String) == 0 ) 
       {
           P38Ports[port-1].parityMode = i;
           break;
       }
       if (i == MAX_PARITY_INDEX)
       {
           sprintf (ConfigFileStr, "-> %.50s %.20s",
                       PortConfig[10].String, string);
           return (3);
       }

   }
   /************************/
   /*  Option number 11    */
   /*  Stop bits.          */
   /************************/


   sprintf (ConfigFileStr, "%s%s", PortConfig[11].String,
               PortConfig[11].Format);
   result = fscanf( file, ConfigFileStr, string );
   if(result != 1)  
   {
       sprintf (ConfigFileStr,"-> %.60s", PortConfig[11].String);
       return( 3 );
   }

   for (i = 0; i < MAX_STOP_BITS_INDEX + 1; i++)
   {
       if( strcmpi( string, StopBitsTable[i].String) == 0 ) 
       {
           P38Ports[port-1].stopBits = i;
           break;
       }
       if (i == MAX_STOP_BITS_INDEX)
       {
           sprintf (ConfigFileStr, "-> %.50s %.20s",
                       PortConfig[11].String, string);
           return (3);
       }

   }

   /************************/
   /*  Option number 12    */
   /*  Rx flow off level   */
   /************************/

   sprintf (ConfigFileStr, "%s%s", PortConfig[12].String,
               PortConfig[12].Format);
   result = fscanf( file, ConfigFileStr, &temp );
   if(result != 1)  
   {
       sprintf (ConfigFileStr,"-> %.60s", PortConfig[12].String);
       return( 3 );
   }
   if (temp > 0 && temp < 1024)
   {
       P38Ports[port-1].rxFlowOffLevel = temp;
   }
   else
   {
       sprintf (ConfigFileStr,"-> %.60s ", PortConfig[12].String);
       return( 3 );
   }

   /************************/
   /*  Option number 13    */
   /*  Rx flow on level    */
   /************************/

   sprintf (ConfigFileStr, "%s%s", PortConfig[13].String,
       PortConfig[13].Format);
   result = fscanf( file, ConfigFileStr, &temp );
   if(result != 1)  
   {
       sprintf (ConfigFileStr,"-> %.60s", PortConfig[13].String);
       return( 3 );
   }

   if (temp > 0 && temp < 1024)
   {
       P38Ports[port-1].rxFlowOnLevel = temp;
   }
   else
   {
       sprintf (ConfigFileStr,"-> %.60s ", PortConfig[13].String);
       return( 3 );
   }

   /************************/
   /*  Option number 14    */
   /*  Rx trigger level    */
   /************************/

   sprintf (ConfigFileStr, "%s%s", PortConfig[14].String,
       PortConfig[14].Format);
   result = fscanf( file, ConfigFileStr, &temp );
   if(result != 1)  
   {
       sprintf (ConfigFileStr,"-> %.60s", PortConfig[14].String);
       return( 3 );
   }

   if (temp > 0 && temp < 1024)
   {
       P38Ports[port-1].rxTrigger = temp;
   }
   else
   {
       sprintf (ConfigFileStr,"-> %.60s ", PortConfig[14].String);
       return( 3 );
   }
  
   /************************/
   /*  Option number 15    */
   /*  Tx trigger level    */
   /************************/
   sprintf (ConfigFileStr, "%s%s", PortConfig[15].String,
       PortConfig[15].Format);
   result = fscanf( file, ConfigFileStr, &temp );
   if(result != 1)  
   {
       sprintf (ConfigFileStr,"-> %.60s", PortConfig[15].String);
       return( 3 );
   }

   if (temp > 0 && temp < 1024)
   {
       P38Ports[port-1].txTrigger = temp;
   }
   else
   {
       sprintf (ConfigFileStr,"-> %.60s ", PortConfig[15].String);
       return( 3 );
   }


   /************************/
   /*  Option number 16    */
   /*  RX IO size          */
   /************************/

   sprintf (ConfigFileStr, "%s%s", PortConfig[16].String,
               PortConfig[16].Format);
   result = fscanf( file, ConfigFileStr, string );
   if(result != 1)  
   {
       sprintf (ConfigFileStr,"-> %.60s", PortConfig[16].String);
       return( 3 );
   }
   if( strcmpi( string, "byte" ) == 0 ) 
       P38Ports[port-1].ReceiveMoveType = BYTE_IO;
   else if( strcmpi( string, "word" ) == 0 ) 
           P38Ports[port-1].ReceiveMoveType = WORD_IO;
        else
        {
           sprintf (ConfigFileStr, "-> %.50s %.20s",
                    PortConfig[16].String, string);
           return (2);
        }

   /************************/
   /*  Option number 17    */
   /*  TX IO size          */
   /************************/

   sprintf (ConfigFileStr, "%s%s", PortConfig[17].String,
               PortConfig[17].Format);
   result = fscanf( file, ConfigFileStr, string );
   if(result != 1)  
   {
       sprintf (ConfigFileStr,"-> %.60s", PortConfig[17].String);
       return( 3 );
   }
   if( strcmpi( string, "byte" ) == 0 ) 
       P38Ports[port-1].TransmitMoveType = BYTE_IO;
   else if( strcmpi( string, "word" ) == 0 ) 
           P38Ports[port-1].TransmitMoveType = WORD_IO;
        else
        {
           sprintf (ConfigFileStr, "-> %.50s %.20s",
                    PortConfig[17].String, string);
           return (2);
        }

   /*******************************/
   /*     Option Number 18        */
   /*     Enhanced IRQ            */  
   /*******************************/

   sprintf (ConfigFileStr, "%s%s", PortConfig[18].String,
               PortConfig[18].Format);
   result = fscanf( file, ConfigFileStr, string );
   if(result != 1)  
   {
       sprintf (ConfigFileStr,"-> %.60s", PortConfig[18].String);
       return( 3 );
   }

   for (temp = 0; temp < 11; temp++)
   {
       if ( strcmpi( string, ESP2IRQ[temp].CfgIRQ ) == 0 ) 
       {
           P38Ports[port-1].IRQValues = &ESP2IRQ[temp];
           if (!(port-1))
               Esp1PIC = P38Ports[port-1].IRQValues->PIC;
           else
               Esp2PIC = P38Ports[port-1].IRQValues->PIC;
           break;
       }
   }
   if (temp >= 11) 
   {
       sprintf (ConfigFileStr, "-> %.50s %.20s",
                 PortConfig[18].String, string);
       return (2);
   }

/***********************************************************************/
/*  ESP2 supports sharing interrupts but this test code is not set up  */
/*  for sharing so will not allow it.                                  */
/***********************************************************************/
   if (port-1)
   {
       if (P38Ports[port-1].IRQValues == P38Ports[0].IRQValues)
       {
           sprintf (ConfigFileStr, "Both ports can NOT run off same IRQ");
           return (2);
       }
   }


   /*******************************/
   /*     Option  number 20       */
   /*     Memory map address      */  
   /*******************************/

   sprintf (ConfigFileStr, "%s%s", PortConfig[20].String,
               PortConfig[20].Format);
   result = fscanf( file, ConfigFileStr, &temp );
   if(result != 1)  
   {
       sprintf (ConfigFileStr,"-> %.60s", PortConfig[20].String);
       return( 3 );
   }

/***********************************************************************/
/* We do not validate memory map address.  We will assume that if we   */
/* are testing we know what memory mapped address to use.              */
/***********************************************************************/

#ifdef DONOTDO
   if (computer == AT)
   {
       for (i = 0; i < MAX_AT_MEMORY_MAP_INDEX; i++)
       {
           if ((WORD) temp == (WORD) MemoryMapAddressAT[i])
           {
               P38Ports[port-1].MemoryMapAddress = (WORD)temp;
               break;
           }
       }
   }
   else
   {
       for (i = 0; i < MAX_MCA_MEMORY_MAP_INDEX; i++)
       {
           if ((WORD) temp == (WORD) MemoryMapAddressMCA[i])
           {
               P38Ports[port-1].MemoryMapAddress = (WORD)temp;
               break;
           }
       }

   }
#endif

   P38Ports[port-1].MemoryMapAddress = (WORD)temp;

   if (!P38Ports[port-1].MemoryMapAddress)
   {
       sprintf (ConfigFileStr,"-> %.60s", PortConfig[20].String);
       return( 3 );
   }

   /*******************************/
   /*     Option 21               */
   /*      DMA channel            */ 
   /*******************************/

   sprintf (ConfigFileStr, "%s%s", PortConfig[21].String,
               PortConfig[21].Format);
   result = fscanf( file, ConfigFileStr, &temp );
   if(result != 1)  
   {
       sprintf (ConfigFileStr,"-> %.60s", PortConfig[21].String);
       return( 3 );
   }
   if (computer == AT)
   {
       if ((temp == 1) || (temp == 3))
           P38Ports[port-1].dmaChannel = (BYTE)temp;
       else
       {
           sprintf (ConfigFileStr,"-> %.60s", PortConfig[21].String);
           return( 3 );
       }
   }
   else
   {
       if ((temp >= 0) || (temp <= 14))
           P38Ports[port-1].dmaChannel = (BYTE)temp;
       else
       {
           sprintf (ConfigFileStr,"-> %.60s", PortConfig[21].String);
           return( 3 );
       }
   }

   /*******************************/
   /*   Option 22                 */
   /*  DMA timeout.               */ 
   /*******************************/
   sprintf (ConfigFileStr, "%s%s", PortConfig[22].String,
               PortConfig[22].Format);
   result = fscanf( file, ConfigFileStr, &temp );
   if(result != 1)  
   {
       sprintf (ConfigFileStr,"-> %.60s", PortConfig[22].String);
       return( 3 );
   }
   P38Ports[port-1].dmaTime = (BYTE)(temp / 10);
   if (P38Ports[port-1].dmaTime > 255)
       P38Ports[port-1].dmaTime = 255;

   /****************************************/
   /*   Option 23                          */
   /*  UART Clock Prescaler (0, 1, 2, 3):  */ 
   /****************************************/

   sprintf (ConfigFileStr, "%s%s", PortConfig[23].String,
               PortConfig[23].Format);
   result = fscanf( file, ConfigFileStr, &temp );
   if(result != 1)  
   {
       sprintf (ConfigFileStr,"-> %.60s", PortConfig[23].String);
       return( 3 );
   }
   if ((temp >= 0) || (temp < 4))
   {
       P38Ports[port-1].UartPrescaler= (BYTE)temp;
   }
   else
   {
       sprintf (ConfigFileStr,"-> %.60s", PortConfig[23].String);
       return( 3 );
   }

   /****************************************/
   /*   Option 24                          */
   /*  Compare test starting character.    */ 
   /****************************************/

   sprintf (ConfigFileStr, "%s%s", PortConfig[24].String,
           PortConfig[24].Format);
   result = fscanf( file, ConfigFileStr, &temp );
   if(result != 1)  
   {
       sprintf (ConfigFileStr,"-> %.60s", PortConfig[24].String);
       return( 3 );
   }
   if ((temp >= 0) || (temp < 0xff))
   {
       CompareStartCharacter[port-1]= (BYTE)temp;
   }
   else
   {
       sprintf (ConfigFileStr,"-> %.60s", PortConfig[24].String);
       return( 3 );
   }


   P38Ports[port-1].PortState = 1;
   return( 0 );
}

/* ================================ GETCONFIG =============================
 * Gets program configuration data from esp.cfg
 * ENTER:       None
 * RETURN:      0=OK, 1=file not found, 2=file format error
 */
int   getconfig(void)
{

   if( ( file = fopen( configfile, "rb" ) ) == NULL )
      return( 1 );

/***********************************************************************/
/*  Get Computer Bus type.  The Computer type field is gathered from   */
/*  byte 6 of the Get System Configuration Paramters.                  */
/***********************************************************************/
   computer = AT;          // assume AT bus
   _asm
   {
       push ax
       push bx
       push es
       mov  ah, 0c0h
       int 15h
       jc  DoesNotSupport
       mov al, es:[bx+5]
       test al, 02
       jz  EndIt
       mov computer, MCA
       jmp     short EndIt
DoesNotSupport:
   }
   printf("\n Cannot get system configuration(INT 15h AH= 0C0h)");
   printf("\n Assuming AT bus type.");
   printf("        Press a key to continue...");
   getch();

   _asm
   {
   EndIt:
       pop es
       pop bx
       pop ax
   }

   /*******************************/
   /*  Port 1 stuff               */
   /*******************************/

   fscanf( file, " Port 1 stuff: " );
   result = get_port_config(1);
   if( result )  
       if (result != 4)
          return( result);

   /*******************************/
   /*  Port 2 stuff               */
   /*******************************/

   fscanf( file, " Port 2 stuff: " );
   result = get_port_config(2);
   if( result )  
   {
       if ( result != 4) 
          return( result);
       else if (( result == 4) &&
               (NumberOfPorts == 0))       // still no ports
                   return (result);
   }

   return( 0 );
}



/* ============================= GETPATTERN =============================
 * Gets test pattern from file esp.tst
 * ENTER:       None
 * RETURN:      1 if file OK, 0 if file not OK
 */
int   getpattern(void)
{
   if(P38Ports[0].PortState)
   {
      if( ( file = fopen( testfile, "rb" ) ) == NULL )
      {
         printf( "Need test file %s\n", testfile );
         return(0);
      }

//      byte_count = fread( tx_buffer[0], sizeof( BYTE ), MAX_COUNT, file );
      byte_count = fread( &save_tx_buffer[0], sizeof( BYTE ), MAX_COUNT, file );
      fclose( file );
      if( byte_count < block_size[0] )
      {
         printf( "Test file %s is too small\n", testfile );
         return(0);
      }
   }

   if(P38Ports[1].PortState)
   {
      if( ( file = fopen( testfile, "rb" ) ) == NULL )
      {
         printf( "Need test file %s\n", testfile );
         return(0);
      }
      byte_count = fread( tx_buffer[1], sizeof( BYTE ), MAX_COUNT, file );
      fclose( file );
      if( byte_count < block_size[1] )
      {
         printf( "Test file %s is too small\n", testfile );
         return(0);
      }
   }
   return(1);
}

/***********************************************************************/
/*     PortDataConfigure                                               */
/*     This routine configures the port from default values and data   */
/*     received from the configuration file ESPTEST.CFG)               */
/***********************************************************************/

void  PortDataConfigure(WORD Port)
{
   WORD        baudDivisor;
   LONG        baudRate;                    
   BYTE        lineCtrlValue;
   WORD        rxTrigger;
   WORD        rxTimeout;
   BYTE        ModemCtrlValue = 0;
   BYTE        svcMask;
   ESPCMDBLOCK cmdBlock;

/***********************************************************************/
/* Disable interrupts for OUR PORT ONLY!                               */
/***********************************************************************/
   _asm    cli   

/***********************************************************************/
/* Set service ID mask but be sure to turn off TX, RX and ERROR ints   */
/* until we are ready to handle them.                                  */
/***********************************************************************/
   svcMask = P38Ports[Port].svcMask;
   svcMask &= ~ESP_SID_PORT1;
   svcMask |=  ESP_SID_ERROR1;
/***********************************************************************/
/* If we are using DMA make sure the DMA interrupt bits get set.       */
/***********************************************************************/
   if ((P38Ports[Port].RxMode == DMA_RX) ||
       (P38Ports[Port].TxMode == DMA_TX))
   { 
       svcMask |= (ESP_SID_DMAFIN | ESP_SID_DMATO);
   } 
   cmdBlock.command = ESP_CMD_SETSVCMASK;
   cmdBlock.ioBase  = P38Ports[Port].EnhancedAddress;
   cmdBlock.writeParms[0] = P38Ports[Port].svcMask = svcMask;
   IssueEspCmdLeaveIntsAlone( &cmdBlock );
/***********************************************************************/
/*     Re-enable interrupts.                                           */
/***********************************************************************/
   _asm    sti             

/***********************************************************************/
/* Calculate baudDivisor.                                              */
/***********************************************************************/
   baudDivisor = BitRateTable[P38Ports[Port].bitRate];

/***********************************************************************/
/* Issue command to set baud rate to baudDivisor.                      */
/***********************************************************************/
   cmdBlock.command = ESP_CMD_SETBAUD;
   cmdBlock.writeParms[0] = HIBYTE(baudDivisor);
   cmdBlock.writeParms[1] = LOBYTE(baudDivisor);
   IssueEspCmd( &cmdBlock );

/***********************************************************************/
/* Set trigger level, which is related to baud rate: trigger every     */
/* 0.1 second.                                                         */
/* the algorithm for the trigger is baud/100 or if > than 19200 (baud) */
/* then the trigger is 256.                                            */
/*                                                                     */
/* OK the above comments indicate that we are using an algorithym but  */
/* during the test cycle we load the triggers via the ESPTEST.CFG file */
/* so that we can test different trigger levels.                       */
/***********************************************************************/
   baudRate = BaudRateTable[ P38Ports[Port].bitRate ];   
   if (baudRate > 19200)
      rxTrigger = 256;
   else if (baudRate <= 100)
      rxTrigger = 1;
   else
      rxTrigger = (WORD) (baudRate / 100);
   cmdBlock.command = ESP_CMD_SETTRIGGER ;
   cmdBlock.writeParms[0] = HIBYTE(P38Ports[Port].rxTrigger);  // first: RX trig MSB
   cmdBlock.writeParms[1] = LOBYTE(P38Ports[Port].rxTrigger);  // next:  RX trig LSB
   cmdBlock.writeParms[2] = HIBYTE(P38Ports[Port].txTrigger);  // next:  TX trig MSB                  
   cmdBlock.writeParms[3] = LOBYTE(P38Ports[Port].txTrigger);  // last:  TX trig LSB

   IssueEspCmd( &cmdBlock );

/***********************************************************************/
/*     Save the trigger value.                                         */
/***********************************************************************/
#ifdef OLDTRIGGERWAY
   P38Ports[Port].rxTrigger = rxTrigger;            
#endif

/***********************************************************************/
/* Set receive timeout to 4 character times, and between 2-239 msec.   */
/*   The following code is ok  for a working timer but our timer is    */
/*   off by a multiple of 10.  I would use a table to get the receive  */
/*   timeout.                                                          */
/*                                                                     */
/*     An example of 4 the character times per second:                 */
/*         Baud = 9600 / 10 = 960 characters per second                */
/*         40000/960 = 41.7 milliseconds for 4 characters in the FIFO  */
/*         We would need to multiply this by ten (417 ms) to handle    */
/*         the first rev of the chip, however, our receive timeout     */
/*         cannot be higher then 255 (byte size) therefore we          */
/*         we use 255.                                                 */        
/***********************************************************************/
   baudRate = (baudRate / 10);
   rxTimeout = (WORD) (40000 / baudRate);

/***********************************************************************/
/*  for testing and for first rev of chip we should multiply by 10     */
/***********************************************************************/
   rxTimeout = (rxTimeout * 10);
   if (rxTimeout > 255)
      rxTimeout = 255;
   else
      if (rxTimeout < 2)
          rxTimeout = 4;

   cmdBlock.command = ESP_CMD_SETRXTIME;
   P38Ports[Port].rxTimeout = cmdBlock.writeParms[0] = (BYTE) rxTimeout;
   IssueEspCmd( &cmdBlock );
/***********************************************************************/
/* Calculate value to be written to ESP Line Control Register.         */
/***********************************************************************/
   lineCtrlValue = 0;
   lineCtrlValue |= DataBitsTable[P38Ports[Port].dataBits].DataBits;
   lineCtrlValue |= StopBitsTable[P38Ports[Port].stopBits].StopBits;
   lineCtrlValue |= ParityTable[P38Ports[Port].parityMode].Parity;
   cmdBlock.command = ESP_CMD_WRITEUART;
   cmdBlock.ioBase = P38Ports[Port].EnhancedAddress;
   cmdBlock.writeParms[0] = UART_LINECTRL;
   cmdBlock.writeParms[1] = lineCtrlValue;
   IssueEspCmd( &cmdBlock );

/***********************************************************************/
/* Set flow control.                                                   */
/***********************************************************************/
   cmdBlock.command = ESP_CMD_SETFCTYPE;
   cmdBlock.ioBase = P38Ports[Port].EnhancedAddress;
   cmdBlock.writeParms[0] = P38Ports[Port].flowCtrlEsp1;
   cmdBlock.writeParms[1] = P38Ports[Port].flowCtrlEsp2;
   IssueEspCmd( &cmdBlock );

   P38Ports[Port].flowCtrlMode = 0;        // clear mode flag

/***********************************************************************/
/* P-38 does not handle the DTR and RTS through the ESP command set so */
/* we need to do it through the UART directly.                         */
/***********************************************************************/
   ModemCtrlValue = 0;
   if (P38Ports[Port].flowCtrlEsp1 & ESP_FC1_RX_DTR)
       ModemCtrlValue = 1;
   if (P38Ports[Port].flowCtrlEsp1 & ESP_FC1_RX_RTS)
       ModemCtrlValue |= 2;

   cmdBlock.command = ESP_CMD_WRITEUART;
   cmdBlock.ioBase = P38Ports[Port].EnhancedAddress;
   cmdBlock.writeParms[0] = UART_MDMCTRL;
   cmdBlock.writeParms[1] = ModemCtrlValue;
   IssueEspCmd( &cmdBlock );

/***********************************************************************/
/* Store away in port instance for later queries.                      */
/***********************************************************************/
   P38Ports[Port].transmitXON  = ASCII_XON;
   P38Ports[Port].transmitXOFF = ASCII_XOFF;

/***********************************************************************/
/* Inform ESP of new characters. Although command takes 5 write        */
/* parameters.                                                         */
/***********************************************************************/
   cmdBlock.command = ESP_CMD_SETFCCHAR;
   cmdBlock.ioBase = P38Ports[Port].EnhancedAddress;
   cmdBlock.writeParms[ 0 ] = ASCII_XON;
   cmdBlock.writeParms[ 1 ] = ASCII_XOFF;
   cmdBlock.writeParms[ 2 ] = 0xFF;
   cmdBlock.writeParms[ 3 ] = 0xFF;
   cmdBlock.writeParms[ 4 ] = 0xFF;
   IssueEspCmd( &cmdBlock );

/***********************************************************************/
/*  Set flow control levels.                                           */
/*     Suggested default values are RX flow off = 768                  */
/*                                  RX flow on  = 512                  */
/***********************************************************************/
   cmdBlock.command = ESP_CMD_SETFCLEVEL ;

   cmdBlock.writeParms[0] = (BYTE) (P38Ports[Port].rxFlowOffLevel / 256);    
   cmdBlock.writeParms[1] = (BYTE) (P38Ports[Port].rxFlowOffLevel % 256);  
   cmdBlock.writeParms[2] = (BYTE) (P38Ports[Port].rxFlowOnLevel  / 256); 
   cmdBlock.writeParms[3] = (BYTE) (P38Ports[Port].rxFlowOnLevel  % 256);  

   IssueEspCmd( &cmdBlock );

/***********************************************************************/
/* Set the error mask.                                                 */
/***********************************************************************/
   cmdBlock.command = ESP_CMD_SETERRMASK;
   cmdBlock.writeParms[0] = ESP_ERR1_RXTO | ESP_ERR1_RXOF | ESP_ERR1_PARITY;
   cmdBlock.writeParms[0] |= ESP_ERR1_FRAME | ESP_ERR1_BRK | ESP_ERR1_MSR;

/***********************************************************************/
/* If either Xon/Xoff or transparent Xon/Xoff is requested we set the  */
/* the onremote and offremote bits in the 2nd tier of error bits.      */
/* This will give us an interrupt so that we can perform Xon/Xoff      */
/* support in our ISR.                                                 */
/*     ***  NOTE:  ****  XON/XOFF hardware control is broken in first  */
/*                       rev of chip.  You need to handle this in      */
/*                       software.                                     */  
/***********************************************************************/
   if (P38Ports[Port].flowCtrlEsp1 & ESP_XO_FLOW_CONTROL)
      cmdBlock.writeParms[1] = ESP_ERR2_ONREMOTE | ESP_ERR2_OFFREMOTE;
   else
      cmdBlock.writeParms[1] = 0;
   IssueEspCmd( &cmdBlock );


/***********************************************************************/
/*  Set UART Prescaler.                                                */
/*     Valid values are:                                               */
/*          0 =  1.8432MHz  Normal                                     */
/*          1 =  3.6864MHz                                             */
/*          2 =  7.3728MHz                                             */
/*          3 = 14.7456MHz                                             */
/***********************************************************************/
   cmdBlock.command = ESP_CMD_SETUARTCLOCK ;
   cmdBlock.writeParms[0] = P38Ports[Port].UartPrescaler; 
   cmdBlock.writeParms[0] |= 4;        /* set wait state IMPORTANT  */
   IssueEspCmd( &cmdBlock );
/***********************************************************************/
/* If AT bus set up the Re-interruptable timer for 4 milliseconds.     */
/***********************************************************************/
   if (computer == AT)
   {
       cmdBlock.writeParms[0] = ESP_DEF_REIRQ_TOUT; 
       cmdBlock.command = ESP_CMD_RIP;
       IssueEspCmd(&cmdBlock);
   }

/***********************************************************************/
/*  Read the SID and error registers to alleviate any confusion.       */
/***********************************************************************/
   inp ((WORD) (P38Ports[Port].EnhancedAddress + (WORD) ESP_CMD_SID));
   cmdBlock.command = ESP_CMD_GETERRS ;   
   cmdBlock.ioBase = P38Ports[Port].EnhancedAddress;
   IssueEspCmdLeaveIntsAlone( &cmdBlock );

}


/* ==========================SetUpHardwareHandler===========================
 * Set up interrupt vector for this port
 * Enter:  BYTE - the port number
 * Return: none
 */

void   SetUpHardwareHandler(BYTE Port)

{
   static  BYTE    CurrentIRQ = 0xFF;
   WORD    Off, Segment;
   BYTE    Vector;


/***********************************************************************/
/* Test for no IRQ or equal to a previous IRQ.  This should not happen */
/* but we will test for it anyway.                                     */
/***********************************************************************/
   if ((P38Ports[Port].IRQValues->ESPIrq == CurrentIRQ) ||
       (P38Ports[Port].IRQValues->ESPIrq == 00))         
   {
       P38Ports[Port].SetVector = FALSE;   /* already set so indicate such */
       return;
   }

/***********************************************************************/
/*     We determine the vector we need.                                */
/***********************************************************************/
   Vector = P38Ports[Port].IRQValues->Vector;

/***********************************************************************/
/*  Get current interrupt vector and save it.                          */
/***********************************************************************/
   _asm
   {
       push    es    
       mov ah, 35h
       mov al, Vector
       int 21h
       mov Off,  bx
       mov bx,es
       mov Segment, bx
       pop    es    
       
   }
   P38Ports[Port].IRQVectorOffset = Off;
   P38Ports[Port].IRQVectorSegment = Segment;

/***********************************************************************/
/*  Set interrupt vector to my ISR.                                    */
/***********************************************************************/
   _asm
   {
       push    ds
       mov ah, 25h
       mov al, Vector
       ;  
       ; this small model progam so picking the CS would work but
       ; you might want to use the FP_SEG() macro to get the CS
       ; of the entry point of the interrupt service routine.  This
       ; would make other models work.
       ;
       mov dx, cs
       mov ds, dx
       mov dx, offset ESPInterrupt1
       cmp Port, 0
       je  SetVector
       mov dx, offset ESPInterrupt2
SetVector:
       int 21h
       pop     ds    
   }

/***********************************************************************/
/*     Indicate that we have set this vector and save the IRQ for later*/
/*     perusal on a subsequent entry.                                  */
/***********************************************************************/
   P38Ports[Port].SetVector = TRUE;
   CurrentIRQ = P38Ports[Port].IRQValues->ESPIrq;

/***********************************************************************/
/* Clear the Mask bit in the IMR register so that it will be hot.      */
/* Turn off interrupts while doing this.                               */
/***********************************************************************/
   _asm    cli             /* clear interrupts */
   outp (P38Ports[Port].IRQValues->MASK, 
           inp (P38Ports[Port].IRQValues->MASK) & 
                   ~(1 << P38Ports[Port].IRQValues->MaskRegisterBit));
/***********************************************************************/
/* restart interrupts.                                                 */
/***********************************************************************/
   _asm    sti             /* start interrupts   */

   return;
}

/* ==========================ReSetIRQVector=================================
 * ReSet Interrupt vector for this port
 * Enter:  none
 * Return: none
 */


void   ResetIRQVector(void)

{
   WORD    Off, Segment, i;
   BYTE    Vector;


/***********************************************************************/
/* Restore the original vectors for all vectors swapped.               */
/***********************************************************************/
   for (i = 0; i < 2; i++)
   {
       if(P38Ports[i].IRQValues->ESPIrq == 00)     // we are in poll mode.
           continue;
       if (P38Ports[i].SetVector == TRUE)
       {
           Vector = P38Ports[i].IRQValues->Vector;
           Off = P38Ports[i].IRQVectorOffset;
           Segment = P38Ports[i].IRQVectorSegment;

           _asm
           {
               push    ds
               mov ah, 25h
               mov al, Vector
               mov dx, Segment
               mov ds, dx
               mov dx, Off
               int 21h
               pop     ds    
           }
/***********************************************************************/
/* Set the Mask bit in the IMR register so that we effectively turn off*/
/* the juice.  Disabel interrupts during.                              */
/***********************************************************************/
           _asm    cli             /* clear interrupts */
           outp (P38Ports[i].IRQValues->MASK, 
                   inp (P38Ports[i].IRQValues->MASK) |
                  (1 << P38Ports[i].IRQValues->MaskRegisterBit));

/***********************************************************************/
/* Restart interrupts.                                                 */
/***********************************************************************/
           _asm    sti             /* start interrupts */
           P38Ports[i].SetVector = FALSE;
       }
   }

   return;
}

/***********************************************************************/
/* SetServiceMask                                                    */
/* Clear the service mask                                              */
/***********************************************************************/

void SetServiceMask (BYTE Port, BYTE Sid)
{
   ESPCMDBLOCK cmdBlock;

   cmdBlock.command = ESP_CMD_SETSVCMASK;    
   cmdBlock.ioBase = P38Ports[Port].EnhancedAddress;
   cmdBlock.writeParms[0] = Sid;
   IssueEspCmdLeaveIntsAlone( &cmdBlock );
   cmdBlock.command = ESP_CMD_GDIPS;         
   IssueEspCmdLeaveIntsAlone( &cmdBlock );

}

/***********************************************************************/
/* SendRemoteCharacter                                                 */
/* Send the flow off character to remote transmitter.                  */
/***********************************************************************/

void SendRemoteCharacter(BYTE Character, BYTE Port)

{

   
   tx_buffer[Port][0] = (BYTE) Character;
   P38Ports[Port].xmtbfrSize = 1;

   SetServiceMask (Port,  0);
   SET (P38Ports[Port].svcMask, ESP_SID_PORT1);  
   SET (P38Ports[Port].flags, FX_TRANSMIT_ON);
   SetServiceMask (Port, P38Ports[Port].svcMask);

}


/* ================================ FindESPPorts =============================
 * Finds and identifies ESP adapters and ports.
 * ENTER:       Port = 1 or 2
 * RETURN:      None
*/

int    FindESPPorts(int Port)
{
   WORD TestResults;
   ESPCMDBLOCK cmdBlock;

/***********************************************************************/
/* Read the BASE IO         for F3h                                    */
/***********************************************************************/
   TestResults = inp(P38Ports[Port].EnhancedAddress);

/***********************************************************************/
/* ESP_2 could have 0xf3 if you read the base I/O                      */
/***********************************************************************/
   if (TestResults == 0xf3)
   {
/***********************************************************************/
/* Do GETTEST and read the products bits for ESP 2.                    */
/***********************************************************************/
       cmdBlock.ioBase = P38Ports[Port].EnhancedAddress;
       cmdBlock.command = ESP_CMD_GETTEST;
       IssueEspCmd( &cmdBlock );
       cmdBlock.uRead.parms[0] &= ESP_2_MASK;
       if ((cmdBlock.uRead.parms[0] >= ESP_2))
       {
           EspAdapterType = ESP_2;
           NumberOfPorts++;
           return (1);
       }
   }

   return (0);

}
