/**************************************************************************
 * ESPDATA.C                                                              *
 * Contains all data for ESP test                                         *
 **************************************************************************/

/*** NOTICE *** NOTICE *** NOTICE *** NOTICE *** NOTICE *** NOTICE ***
 ***                                                               ***
DISCLAIMER: THE PRODUCT IS PROVIDED TO DEVELOPER "AS IS" WITHOUT WARRANTY
OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE 
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE,
AND THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PRODUCT IS 
ASSUMED BY DEVELOPER, INCLUDING ALL COSTS OF SERVICING, REPAIRS OR CORRECTIONS.  
DEVELOPER AGREES THAT HAYES WILL NOT BE LIABLE FOR ANY DAMAGES DIRECTLY OR 
INDIRECTLY RELATED TO THE USE OF THE PRODUCT, INCLUDING BUT NOT LIMITED
TO, LOST PROFITS, INDIRECT, SPECIAL OR CONSEQUENTIAL DAMAGES, WHETHER BASED
ON CONTRACT, TORT OR ANY OTHER LEGAL THEORY.  DEVELOPER ACKNOWLEDGES THAT 
THE PRODUCT IS STILL UNDER DEVELOPMENT AND THAT HAYES DOES NOT WARRANT 
THAT THE PRODUCT IN ITS PRESENT OR FUTURE FORM WILL MEET DEVELOPER'S NEEDS.
 ***                                                               ***
 *** NOTICE *** NOTICE *** NOTICE *** NOTICE *** NOTICE *** NOTICE ***/

/***********************************************************************/
/*  Copyright 1993 Hayes Microcomputer Products, Inc.                  */
/*  All rights reserved.                                               */
/*  Use of copyright notice does not imply publication.                */ 
/***********************************************************************/
/***********************************************************************/
/*                                                                     */
/*                                                                     */
/*     NOTE:   This code is specifically for the ESP2 board.           */
/*                                                                     */
/*                     T E S T    C O D E                              */
/*                                                                     */
/*     This code uses the Hayes Enhanced Serial Interface (ESI) Ver 2  */    
/*     to operate the ESP2 adapter.  Please refer to that manual for   */
/*     more detail on the commands.                                    */
/***********************************************************************/


#include <stdio.h>
#include <conio.h>
#include <memory.h>
#include <stdlib.h>
#include <string.h>
#include <dos.h>
#include "esptype.h"
#include "espfunc.h"


/***********************************************************************/
/*     Global Data items.                                              */
/***********************************************************************/
BYTE   computer;
PCHAR  configfile = "esptest.cfg";
FILE   *file;
BYTE   dma_time;
double TotalTXbytes[2] = {0.,0.};
double TotalRXbytes[2] = {0.,0.};
BYTE   CompareStartCharacter[2] = {0x14, 0x14};  
INT    block_size[2];
BYTE   rx_buffer[2][MAX_RECV_COUNT];
BYTE   tx_buffer[2][MAX_COUNT];
BYTE   save_tx_buffer[MAX_COUNT];
PCHAR  testfile = "esptest.dat";
INT    result;
INT    byte_count;
INT    i;   
INT    key = '1';
INT    quit = 0;
BYTE   IOWait = 0;           /* 0 = wait,  1 = nowait    */
BYTE   EspAdapterType = 0;
CHAR   ConfigFileStr[81];
BYTE   NumberOfPorts = 0;  

/***********************************************************************/
/* This is the table that id's the configuration text and entries.     */
/* NOTE:  This must stay identical to what is in the ESPTEST.CFG file. */
/***********************************************************************/
struct PortConfiguration PortConfig[]  =
{
   NULL,NULL,
   " Activate Port (Y or N): ","%s",
   " Enhanced address (hex): ","%x",
   " Flow Control Byte 1 (hex): ","%x", 
   " Flow Control Byte 2 (hex): ","%x", 
   " RX transfer type (MEMMAP, IO or DMA): ","%s",
   " TX transfer type (MEMMAP, IO or DMA): ","%s",
   NULL,NULL,
   " Baud: ","%ld",
   " Data bits (5, 6, 7, or 8): ","%d",
   " Parity (NONE, ODD, EVEN, MARK, or SPACE): ","%s",
   " Stop bits (1, 1.5, or 2): ","%s",
   " Rx Flow Off Level (1-1023): ","%d",
   " Rx Flow On Level (1-1023): ","%d",
   " Rx Trigger Level (1-1023): ","%d",
   " Tx Trigger Level (1-1023): ","%d",
   " RX IO Size (BYTE or WORD): ","%s",
   " TX IO Size (BYTE or WORD): ","%s",
   " Enhanced IRQ (9,3,4,5,7,10,11,12,14,15,POLL): ","%s",
   NULL,NULL,
   " Memory map address (hex): ","%x",
   " DMA channel (1 or 3): ","%d",
   " DMA timeout (ms): ","%d",
   " UART Clock Prescaler (0, 1, 2, 3): ","%d",
   " Packet Compare Start Character (Hex 00-FE): ", "%x",
   NULL
};

/***********************************************************************/
/* IRQ table.  Contains  the value comparison, IRQ value, Vector entry */
/*             PIC, PIC mask, DMA IRQ for ESP and Masking bit for IMR  */
/***********************************************************************/
struct   IRQS  ESP2IRQ [] =
{
   "9", 0x02, 0x71, CTRL_REG_SLAVE_8259, MASK_REG_SLAVE_8259, 0x01, 0x01,
   "3", 0x03, 0x0b, CTRL_REG_8259, MASK_REG_8259, 0x03, 0x03,
   "4", 0x04, 0x0c, CTRL_REG_8259, MASK_REG_8259, 0x05, 0x04,
   "5", 0x05, 0x0d, CTRL_REG_8259, MASK_REG_8259, 0x07, 0x05,
   "poll", 0, 0, 0, 0, 0, 0,
   "7", 0x07, 0x0f, CTRL_REG_8259, MASK_REG_8259, 0x01, 0x07,
   "10", 0x0a, 0x72, CTRL_REG_SLAVE_8259, MASK_REG_SLAVE_8259, 0x01, 0x02,
   "11", 0x0b, 0x73, CTRL_REG_SLAVE_8259, MASK_REG_SLAVE_8259, 0x01, 0x03,
   "12", 0x0c, 0x74, CTRL_REG_SLAVE_8259, MASK_REG_SLAVE_8259, 0x01, 0x04,
   "14", 0x0e, 0x76, CTRL_REG_SLAVE_8259, MASK_REG_SLAVE_8259, 0x01, 0x06,
   "15", 0x0f, 0x77, CTRL_REG_SLAVE_8259, MASK_REG_SLAVE_8259, 0x01, 0x07,
   NULL, 0, 0 ,0, 0, 0, 0
};

/***********************************************************************/
/*     Bit rate table                                                  */
/***********************************************************************/
WORD BitRateTable[MAX_BIT_RATE_INDEX+1] =
   { 
         0x0900, 0x0600, 0x0417, 0x0359,  //     50     75    110    134.5  
         0x0300, 0x0180, 0x00C0, 0x0060,  //    150    300    600   1200   
         0x0040, 0x003A, 0x0030, 0x0020,  //   1800   2000   2400   3600   
         0x0018, 0x0010, 0x000C, 0x0006,  //   4800   7200   9600  19200   
         0x0003, 0x0002, 0x0001           //  38400  57600 115200
   };

/***********************************************************************/
/* Baud rate table.                                                    */
/***********************************************************************/
LONG BaudRateTable[MAX_BIT_RATE_INDEX+1] =
   { 
         50, 75, 110, 134,                //     50     75    110    134.5  
         150, 300, 600, 1200,             //    150    300    600   1200   
         1800, 2000, 2400, 3600,          //   1800   2000   2400   3600   
         4800, 7200, 9600, 19200,         //   4800   7200   9600  19200   
         38400, 57600, 115200             //  38400  57600 115200
   };

/***********************************************************************/
/* Parity table.                                                       */
/***********************************************************************/
struct PARITY ParityTable[] =
   {
         "none",   0x00,           // no parity 
         "odd",    0x08,           // odd parity 
         "even",   0x18,           // even parity 
         "mark",   0x28,           // mark parity 
         "space",  0x38            // space parity 
   };
/***********************************************************************/
/* stop bits table.                                                    */
/***********************************************************************/
struct STOPBITS StopBitsTable[] =
   {
       "1",    0x00,       // 1 stop bit 
       "1.5",  0x04,       // 1.5 stop bits, substitute 2 stop bits if not 5 bits/char
       "2",    0x04        // 2 stop bits 
   };

/***********************************************************************/
/* Data bits table.                                                    */
/***********************************************************************/
struct DATABITS DataBitsTable[] =
   {
      0x05, 0x00,         // 5 bits 
      0x06, 0x01,         // 6 bits 
      0x07, 0x02,         // 7 bits 
      0x08, 0x03          // 8 bits 
   };

/***********************************************************************/
/* Memory map address table for AT bus.                                */
/***********************************************************************/
WORD MemoryMapAddressAT[MAX_AT_MEMORY_MAP_INDEX] =
   { 
         0xc000, 0xc400, 0xc800, 0xcc00,               
         0xd000, 0xd400, 0xd800, 0xdc00            
   };

/***********************************************************************/
/* Memory map address table for MCA bus.                               */
/***********************************************************************/
WORD MemoryMapAddressMCA[MAX_MCA_MEMORY_MAP_INDEX] =
   { 
         0xc000, 0xc200, 0xc400, 0xc600, 0xc800, 0xca00, 0xcc00, 0xce00,
         0xd000, 0xd200, 0xd400, 0xd600, 0xd800, 0xda00, 0xdc00, 0xde00            
   };

/***********************************************************************/
/* Enhanced IO address table for AT bus.                               */
/***********************************************************************/
WORD EnhancedAddressAT[MAX_AT_ENH_ADDRESS_INDEX] =
   { 
         0x100, 0x140, 0x180, 0x200, 
         0x240, 0x280, 0x300, 0x380
   };

/***********************************************************************/
/* Enhanced IO address table for MCA bus.                              */
/***********************************************************************/
WORD EnhancedAddressMCA[MAX_MCA_ENH_ADDRESS_INDEX] =
   { 
         0x8100, 0x8140, 0x8180, 0x8200, 
         0x8240, 0x8280, 0x8300, 0x8380
   };

/***********************************************************************/
/* State table for each port.                                          */
/***********************************************************************/
struct   ESP38 P38Ports[2] =      
{
   {0},
   {0}
};

