.286p

;******************************************************************;
;*                                                                *;      
;DISCLAIMER: THE PRODUCT IS PROVIDED TO DEVELOPER "AS IS" WITHOUT WARRANTY
;OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE 
;IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE,
;AND THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PRODUCT IS 
;ASSUMED BY DEVELOPER, INCLUDING ALL COSTS OF SERVICING, REPAIRS OR CORRECTIONS.  
;DEVELOPER AGREES THAT HAYES WILL NOT BE LIABLE FOR ANY DAMAGES DIRECTLY OR 
;INDIRECTLY RELATED TO THE USE OF THE PRODUCT, INCLUDING BUT NOT LIMITED
;TO, LOST PROFITS, INDIRECT, SPECIAL OR CONSEQUENTIAL DAMAGES, WHETHER BASED
;ON CONTRACT, TORT OR ANY OTHER LEGAL THEORY.  DEVELOPER ACKNOWLEDGES THAT 
;THE PRODUCT IS STILL UNDER DEVELOPMENT AND THAT HAYES DOES NOT WARRANT 
;THAT THE PRODUCT IN ITS PRESENT OR FUTURE FORM WILL MEET DEVELOPER'S NEEDS.
;******************************************************************;
;******************************************************************;
;*  ESPENTRY.ASM  - This is the ESP strategy entry points.         ;
;*                                                                *;            
;*  CHANGE HISTORY:                                               *;
;*                11-23-92 - Created                              *;
;******************************************************************;


;******************************************************************;
;*      The _DATA segment needs to be first in order so that the   ;
;*      the device header is at the beginning of the image.        ;
;*      Make sure this file is the first in the link.              ;
;******************************************************************;

_DATA   segment word    public  'DATA'
_DATA   ends

_TEXT   segment word public 'CODE'
_TEXT   ends

CONST   segment word    public  'CONST'
CONST   ends

_BSS    segment word    public  'BSS'
_BSS    ends


DGROUP  group   CONST,  _BSS,   _DATA

;******************************************************************;
;*      Public Declarations
;******************************************************************;
        PUBLIC  _ESPInterrupt1
        PUBLIC  _ESPInterrupt2

;******************************************************************;
;*      Extern Declarations
;******************************************************************;
        extrn   _ESPIsr:near
        extrn   _RentrantFlag:byte
CARRY                   equ     1
MASK_REG_8259           equ     0x21
MASK_REG_SLAVE_8259     equ     0xA1
CTRL_REG_8259           equ     0x20
CTRL_REG_SLAVE_8259     equ     0xA0
EOI                     equ     0x20


;******************************************************************;
; Start of DATA segment
;******************************************************************;
_DATA   segment word    public  'DATA'

;******************************************************************;
; This is the PIC value for the first ESP entry point.             
;******************************************************************;
PUBLIC  _Esp1PIC
_Esp1PIC     db  ?

;******************************************************************;
; This is the PIC value for the second ESP entry point.             
;******************************************************************;
PUBLIC  _Esp2PIC
_Esp2PIC     db  ?

;******************************************************************;
; This is the STACK for the first ESP entry point.             
;******************************************************************;
StackESP1   db  512    dup ('1')
StartStackeESP1 label   byte

;******************************************************************;
; This is the STACK for the second ESP entry point.             
;******************************************************************;
StackESP2   db  512    dup ('2')
StartStackeESP2 label   byte

_DATA   ends


;******************************************************************;
; Start of CODE segment
;******************************************************************;
_TEXT   segment word public 'CODE'

        assume  ds:DGROUP, es:NOTHING, ss:NOTHING, cs:_TEXT

;******************************************************************;
; The following fields were store in code segment so that we
; minimize the use of somebody else's stack upon entry in the ISR
;******************************************************************;
        SS_Esp1     dw  ?
        SP_Esp1     dw  ?
        Busy1       db  0
        SS_Esp2     dw  ?
        SP_Esp2     dw  ?
        Busy2       db  0
        SaveAX1     dw  16 dup ('C')
        SaveAX2     dw  16 dup ('C')
        

;******************************************************************;
;       Interrupts for ESP 1 come through here
;******************************************************************;
_ESPInterrupt1        proc    far      
        assume  ds:DGROUP, es:NOTHING, ss:NOTHING, cs:_TEXT

;******************************************************************;
; Push DS on stack
;******************************************************************;
        push    ds

;******************************************************************;
; turn off interrupts until we set up house
;******************************************************************;
        cli
;******************************************************************;
; We save AX in our code segment so that we can use ax until the
; end and we do not affect the stack
;******************************************************************;
        push    bx
        mov     bl,cs:busy1
        xor     bh,bh
        shl     bx,1
        mov     cs:SaveAX1[bx], ax
        pop     bx
        inc     cs:busy1
;******************************************************************;
;   Save current stack and set up our own
;******************************************************************;
        mov     ax, DGROUP
        mov     ds, ax
        mov     cs:SS_Esp1, ss
        mov     cs:SP_Esp1, sp
        mov     ss,ax
        mov     sp,offset StartStackeESP1
;******************************************************************;
;   Turn interrupts back on 
;******************************************************************;
        sti

;******************************************************************;
;   Push all registers before we go into our 'C' ISR
;******************************************************************;
        push    bx
        push    cx
        push    dx
        push    di
        push    si
        push    bp
        push    es
;******************************************************************;
;   Get DS and ES to equal our data segment 
;******************************************************************;
        mov     ax, DGROUP
        mov     ds, ax
        mov     es, ax

;******************************************************************;
;   Push a 0 on the stack which tells our ISR that this is Port 1
;******************************************************************;
        push    0
;******************************************************************;
;       Call our ISR (ESPISR.C) 
;******************************************************************;
        call    _ESPIsr               ; call common ISR

        add     sp, 2                 ; clean up stack

;******************************************************************;
;  Restore registers from our private stack
;******************************************************************;
        pop     es
        pop     bp
        pop     si
        pop     di
        pop     dx
        pop     cx
        pop     bx

;******************************************************************;
; turn off interrupts and restore the system stack 
;******************************************************************;
        cli                           ; restore stack
        mov     ss, cs:SS_Esp1
        mov     sp, cs:SP_Esp1
;******************************************************************;
;  restart interrupts. 
;******************************************************************;
        sti

;******************************************************************;
; Determine what PIC we use and EOI it. 
;******************************************************************;
        mov     al, 20h                 ; set eoi value
        cmp     _Esp1PIC, 0a0h          ; see what PIC we are working
        pop     ds                      ; restore the last registers

        jne     DoMaster1
        out     0a0h, al

DoMaster1:
        out     20h, al

;******************************************************************;
;   Turn off interrupts and restore AX properly
;******************************************************************;
        cli
        dec     cs:busy1
        push    bx
        mov     bl,cs:busy1
        xor     bh,bh
        shl     bx,1
        mov     ax, cs:SaveAX1[bx]
        pop     bx
        sti

;******************************************************************;
;  Go back to the system we are finished      
;******************************************************************;
GetOut1:
        iret

_ESPInterrupt1        endp



_ESPInterrupt2        proc    far      
        assume  ds:DGROUP, es:NOTHING, ss:NOTHING, cs:_TEXT

;******************************************************************;
; Push DS on stack
;******************************************************************;
        push    ds

;******************************************************************;
; turn off interrupts until we set up house
;******************************************************************;
        cli
;******************************************************************;
; We save AX in our code segment so that we can use ax until the
; end and we do not affect the stack
;******************************************************************;
        push    bx
        mov     bl,cs:busy2
        xor     bh,bh
        shl     bx,1
        mov     cs:SaveAX2[bx], ax
        pop     bx
        inc     cs:busy2
;******************************************************************;
;   Save current stack and set up our own
;******************************************************************;
        mov     ax, DGROUP
        mov     ds, ax
        mov     cs:SS_Esp2, ss
        mov     cs:SP_Esp2, sp
        mov     ss,ax
        mov     sp,offset StartStackeESP2
;******************************************************************;
;   Turn interrupts back on 
;******************************************************************;
        sti

;******************************************************************;
;   Push all registers before we go into our 'C' ISR
;******************************************************************;
        push    bx
        push    cx
        push    dx
        push    di
        push    si
        push    bp
        push    es
;******************************************************************;
;   Get DS and ES to equal our data segment 
;******************************************************************;
        mov     ax, DGROUP
        mov     ds, ax
        mov     es, ax

;******************************************************************;
;   Push a 1 on the stack which tells our ISR that this is Port 2
;******************************************************************;
        push    1
;******************************************************************;
;       Call our ISR (ESPISR.C) 
;******************************************************************;
        call    _ESPIsr               ; call common ISR

        add     sp, 2                 ; clean up stack

;******************************************************************;
;  Restore registers from our private stack
;******************************************************************;
        pop     es
        pop     bp
        pop     si
        pop     di
        pop     dx
        pop     cx
        pop     bx

;******************************************************************;
; turn off interrupts and restore the system stack 
;******************************************************************;
        cli                           ; restore stack
        mov     ss, cs:SS_Esp2
        mov     sp, cs:SP_Esp2
;******************************************************************;
;  restart interrupts. 
;******************************************************************;
        sti

;******************************************************************;
; Determine what PIC we use and EOI it. 
;******************************************************************;
        mov     al, 20h                 ; set eoi value
        cmp     _Esp2PIC, 0a0h          ; see what PIC we are working
        pop     ds                      ; restore the last registers

        jne     DoMaster2
        out     0a0h, al

DoMaster2:
        out     20h, al

;******************************************************************;
;   Turn off interrupts and restore AX properly
;******************************************************************;
        cli
        dec     cs:busy2
        push    bx
        mov     bl,cs:busy2
        xor     bh,bh
        shl     bx,1
        mov     ax, cs:SaveAX2[bx]
        pop     bx
        sti

;******************************************************************;
;  Go back to the system we are finished      
;******************************************************************;

GetOut2:
        iret

_ESPInterrupt2        endp


_TEXT   ends
        end
