/**************************************************************************
 * ESPFUNC.H                                                              *
 * Declares the ESP interface functions                                   *
 **************************************************************************/

/*** NOTICE *** NOTICE *** NOTICE *** NOTICE *** NOTICE *** NOTICE ***
 ***                                                               ***
DISCLAIMER: THE PRODUCT IS PROVIDED TO DEVELOPER "AS IS" WITHOUT WARRANTY
OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE 
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE,
AND THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PRODUCT IS 
ASSUMED BY DEVELOPER, INCLUDING ALL COSTS OF SERVICING, REPAIRS OR CORRECTIONS.  
DEVELOPER AGREES THAT HAYES WILL NOT BE LIABLE FOR ANY DAMAGES DIRECTLY OR 
INDIRECTLY RELATED TO THE USE OF THE PRODUCT, INCLUDING BUT NOT LIMITED
TO, LOST PROFITS, INDIRECT, SPECIAL OR CONSEQUENTIAL DAMAGES, WHETHER BASED
ON CONTRACT, TORT OR ANY OTHER LEGAL THEORY.  DEVELOPER ACKNOWLEDGES THAT 
THE PRODUCT IS STILL UNDER DEVELOPMENT AND THAT HAYES DOES NOT WARRANT 
THAT THE PRODUCT IN ITS PRESENT OR FUTURE FORM WILL MEET DEVELOPER'S NEEDS.
 ***                                                               ***
 *** NOTICE *** NOTICE *** NOTICE *** NOTICE *** NOTICE *** NOTICE ***/

/***********************************************************************/
/*    Copyright 1993 Hayes Microcomputer Products, Inc.                */
/*    All rights reserved.                                             */
/*    Use of copyright notice does not imply publication.              */
/***********************************************************************/
/***********************************************************************/
/*                                                                     */
/*                                                                     */
/*     NOTE:   This code is specifically for the ESP2 board.           */
/*                                                                     */
/*                     T E S T    C O D E                              */
/*                                                                     */
/*     This code uses the Hayes Enhanced Serial Interface (ESI) Ver 2  */    
/*     to operate the ESP2 adapter.  Please refer to that manual for   */
/*     more detail on the commands.                                    */
/***********************************************************************/

/********************************* DEFINES ********************************/
#define  READ              1
#define  WRITE             0
#define  AT                0
#define  MCA               1
#define  TRUE              1
#define  FALSE             !TRUE
#define  MAX_COUNT         1024
#define  MAX_RECV_COUNT    20 * 1024
//#define  MAX_RECV_COUNT    1024

#define  MAX_AT_MEMORY_MAP_INDEX   8
#define  MAX_MCA_MEMORY_MAP_INDEX  16
#define  MAX_AT_ENH_ADDRESS_INDEX  8
#define  MAX_MCA_ENH_ADDRESS_INDEX  8
#define  MAX_COUNT   1024
#define  NONE        0
#define  ECHO        1
#define  LOOPBACK    2
#define  RX_TRIGGER  256
#define  RX_TRIGGER_19200  255
#define  TX_TRIGGER  768


#define MIN_BIT_RATE_INDEX      (0)   
#define MAX_BIT_RATE_INDEX      (18)
#define MIN_DATA_BITS_INDEX     (0)
#define MAX_DATA_BITS_INDEX     (3)
#define MIN_STOP_BITS_INDEX     (0)
#define MAX_STOP_BITS_INDEX     (2)
#define MIN_PARITY_INDEX        (0)
#define MAX_PARITY_INDEX        (4)
#define MIN_FLOWCTRL_INDEX      (0|0)  
#define MAX_FLOWCTRL_INDEX      (1|2)

#define NOT_SUPPORTED             0
#define ASCII_XON   (0x11)
#define ASCII_XOFF  (0x13)

/***********************************************************************/
/* PIC defines                                                         */
/***********************************************************************/
#define MASK_REG_8259         0x21
#define MASK_REG_SLAVE_8259   0xA1
#define CTRL_REG_8259         0x20
#define CTRL_REG_SLAVE_8259   0xA0
#define EOI                   0x20

/***********************************************************************/
/* MODE BIT DEFINITIONS                                                */
/***********************************************************************/
#define  UNENHANCED        0x00
#define  ENHANCED          0x01
#define  CHAR_RX           0x00
#define  DMA_RX            0x10
#define  CHAR_TX           0x00
#define  DMA_TX            0x20
#define  MEMORY_MAPPED     0x40
#define  BYTE_IO           0x00
#define  WORD_IO           0x01

/***********************************************************************/
/* FLOW CONTROL BITS                                                   */
/***********************************************************************/
#define  RX_XX_FLOW        0x01
#define  TXX_FLOW          0x02
#define  RX_RTS_FLOW       0x04
#define  RX_DTR_FLOW       0x08
#define  RTS               0x10
#define  DTR               0x20
#define  LOOP              0x40
#define  TX_XX_FLOW        0x80
#define  RX_CTS_FLOW       0x10
#define  RX_DSR_FLOW       0x20

/***********************************************************************/
/* UART REGISTER ADDRESSES                                             */
/***********************************************************************/
#define  THR               0x0
#define  IIR               0x2
#define  LCR               0x3
#define  MCR               0x4
#define  LSR               0x5
#define  SCR               0x7
#define  DLL               0x0
#define  DLM               0x1

/***********************************************************************/
/* STATUS BITS                                                         */
/***********************************************************************/
#define  PE                0x0400
#define  FE                0x0800
#define  BI                0x1000

/***********************************************************************/
/* REGISTER READY BITS                                                 */
/***********************************************************************/
#define  RX1_RDY           0x01
#define  TX1_RDY           0x02
#define  RX2_RDY           0x04
#define  TX2_RDY           0x08
#define  STATUS1_RDY       0x10
#define  COMMAND1_RDY      0x20
#define  STATUS2_RDY       0x40
#define  COMMAND2_RDY      0x80

/***********************************************************************/
/* SERVICE ID BITS                                                     */
/***********************************************************************/
#define  ESP_CMD_SID       0x01
#define  RX1_FIFO_SR       0x01
#define  TX1_FIFO_SR       0x02
#define  P1_ERROR          0x04
#define  DMA_TC_REACHED    0x08
#define  RX2_FIFO_SR       0x10
#define  TX2_FIFO_SR       0x20
#define  P2_ERROR          0x40
#define  DMA_TIMEOUT       0x80

#define  TEMT              0x0200
#define  RX_TIMEOUT        0x0100

/***********************************************************************/
/*         DMA Status bits                                             */
/***********************************************************************/
#define    DMAPORT1        0x01
#define    DMAPORT2        0x02
#define    DMAREAD         0x04
#define    DMAWRITE        0x08

/*************************  ESP-2 P/38  *******************************/
/*  Get self test bits  CMD1 = 01h */
/*  Bit    76543210                */
/*         1.......    Slave       */
/*         0.......    Master      */
/*         .010....    ESP-2 ID    */
/*         .001....    ESP-1 ID    */
/*         ....0...    AT-Bus      */
/*         ....1...    MCA- BUS    */
/*         .....000    Reserved    */
/***********************************/
#define    GET_SELF_TEST   0x01
#define    SLAVE           0x80
#define    MASTER          0x00
#define    ESP_2           0x20
#define    ESP_2_MASK      0x70
#define    ESP_1           0x10
#define    AT_BUS          0x00
#define    MCA_BUS         0x08

/***************************************/
/* Get Normal Mode address CMD1 = 02h  */
/* Bit     76543210                    */
/*         0.......    UART Compatible */  
/*         1.......    Enhanced mode   */
/*         ..00....    COM1, IRQ4      */
/*         ..01....    COM2, IRQ3      */
/*         ..10....    COM3, IRQ4      */
/*         ..11....    COM4, IRQ3      */
/*         ......00    COM1, IRQ4      */
/*         ......01    COM2, IRQ3      */
/*         ......10    COM3, IRQ4      */
/*         ......11    COM4, IRQ3      */
/*         .0..00..    Reserved        */         
/***************************************/
#define    GET_NORMAL_DIP  0x02
#define    UART_COMPATIBLE 0x00
#define    ENHANCED_ESP2   0x80
#define    PORT1_COM1_IRQ4 0x00
#define    PORT1_COM2_IRQ3 0x01
#define    PORT1_COM3_IRQ4 0x02
#define    PORT1_COM4_IRQ3 0x03
#define    PORT2_COM1_IRQ4 0x00
#define    PORT2_COM2_IRQ3 0x10
#define    PORT2_COM3_IRQ4 0x20
#define    PORT2_COM4_IRQ3 0x30

/***************************************/
/* Set Error Mask CMD1 = 02h           */
/* Get Error Mask CMD1 = 12h STAT 1+2  */
/*  Set's available ESP IRQ's          */
/* Bit     76543210                    */
/*         1.......    Start TX break  */
/*         .1......    Flow off timeout*/
/*         ..1.....    Modem Status chg*/
/*         ...1....    RX break        */
/*         ....1...    framing error   */
/*         .....1..    parity error    */
/*         ......1.    rx fifo overflow*/
/*         .......1    rx char. timeout*/
/***************************************/
/* Set Error Mask CMD2 =               */
/*  Set's available ESP IRQ's          */
/* Bit     76543210                    */
/*         ....1...    Rem TX flow on  */
/*         .....1..    Loc TX flow on  */
/*         ......1.    Loc TX flow off */
/*         .......1    Rem TX flow off */
/***************************************/
#define    SET_ERROR_MASK  0x07
#define    START_TX_BREAK  0x80
#define    FLOW_OFF_TO     0x40
#define    MODEM_STATUS    0x20
#define    RX_BREAK        0x10
#define    FRAMING_ERROR   0x08
#define    PARITY_ERROR    0x04
#define    RX_FIFO_OFLOW   0x02
#define    RX_CHAR_TO      0x01
#define    REMOTE_TX_FLON  0x08
#define    LOCAL_TX_FLON   0x04
#define    LOCAL_TX_FLOFF  0x02
#define    REMOTE_TX_FLOFF 0x01


/***************************************/
/* Set Mode CMD1 = 10h                 */
/* Get MODE (1Eh) STAT2                */
/* CMD2 =                              */
/* Bit     76543210                    */
/*         1.......    Force 1k buffer */
/*         0.......    normal fifo ctl */
/*         .1......    mapmem addr ena */
/*         ..1.....    DMA enabled     */
/*         ...1....    DMA enabled     */
/*         ....1...    comp. DTR hw ena*/
/*         .....1..    comp. RTS hw ena*/
/*         ......1.    FIFO enabled    */
/*         .......1    Enhanced        */
/*   See Notes on P-38 to determine    */
/*   how these bit settings affect each*/
/*   other.                            */
/***************************************/
#define    SET_MODE_CMD    0x07
#define    FIFO_1K         0x80
#define    FIFO_NORMAL     0x00
#define    ENABLE_MAPMEM   0x40
#define    DMA_ENABLED_TX  0x20
#define    DMA_ENABLED_RX  0x10
#define    COMP_DTR_HW_ENA 0x08
#define    COMP_RTS_HW_ENA 0x04
#define    FIFO_ENABLED    0x02
#define    MODE_ENHANCED   0x01
#define    MODE_COMPATIBLE 0x00

/***************************************/
/* Get Mode CMD1 = 1Eh                 */
/* Get MODE     STAT1                  */
/* Bit     76543210                    */
/*         00......    1.8432MHz       */
/*         01......    3.6864MHz       */
/*         10......    7.3728MHz       */
/*         11......    14.7456MHz      */
/*         ..000000    Reserved        */
/***************************************/
#define    GET_MODE        0x1E
#define    CLK18432        0x00
#define    CLK36864        0x40
#define    CLK73728        0x80
#define    CLK14745        0xC0

/***************************************/
/* Issue Line Break = 1Ah              */
/* CMD2 =                              */
/* Bit     76543210                    */
/*         .......1    Set break       */
/*         .......0    Clear break     */
/***************************************/
#define    ISSUE_LINE_BRK  0x1A
#define    SET_BREAK       0x01
#define    CLEAR_BREAK     0x00

/***************************************/
/* Set re-interrupt pacing = 20h       */
/* 1 to 255 ticks of 100uS clock       */
/* AT-BUS only                         */
/***************************************/
#define    RE_INTERRUPT    0x20

/***************************************/
/* Set UART Clock Prescaler = 23h      */
/* CMD2 =                              */
/*     D = 1.8432                      */
/* Bit     76543210                    */
/*         ......00    D * 1 = 1.8432  */
/*         ......01    D * 2 = 3.6864  */
/*         ......10    D * 4 = 7.3728  */
/*         ......11    D * 8 = 14.7456 */
/***************************************/
#define    SET_UART_CLOCK  0x23        
#define    SET_18432       CLK18432 >> 6
#define    SET_36864       CLK36864 >> 6
#define    SET_73728       CLK73728 >> 6
#define    SET_14745       CLK14745 >> 6

/***************************************/
/* Set Enhanced Interrupt Level = 1fh  */
/* CMD2 =                              */
/* Bit     76543210                    */
/*         ......1.    IRQ9 (2)        */
/*         ......11    IRQ3            */
/*         .....1..    IRQ4            */
/*         .....1.1    IRQ5            */
/*         .....111    IRQ7            */
/*         ....1.1.    IRQ10           */
/*         ....1.11    IRQ11           */
/*         ....11..    IRQ12           */
/*         ....11.1    IRQ13 (MCA)     */
/*         ....111.    IRQ14           */
/*         ....1111    IRQ15           */
/***************************************/
#define    SET_ENHANCED_IRQ    0x1F
#define    ENH_IRQ9    0x02
#define    ENH_IRQ3    0x03
#define    ENH_IRQ4    0x04
#define    ENH_IRQ5    0x05
#define    ENH_IRQ7    0x07
#define    ENH_IRQ10   0x0a
#define    ENH_IRQ11   0x0b
#define    ENH_IRQ12   0x0c
#define    ENH_IRQ13   0x0d
#define    ENH_IRQ14   0x0e
#define    ENH_IRQ15   0x0f

/***********************************************************************/
/*             Structures                                              */
/***********************************************************************/
#pragma pack(1)
/***********************************************************************/
/*     P-38 state variable structure for a port.                       */
/***********************************************************************/
struct   ESP38 
{
   WORD    EnhancedAddress;
   WORD    MemoryMapAddress;
   WORD    IRQVectorOffset;
   WORD    IRQVectorSegment;
   WORD    DMAStatus;
   WORD    flags;
   ULONG   receiveBytes;
   ULONG   transmitBytes;
   ULONG   parityErrors;
   ULONG   framingErrors;
   ULONG   ISREntry;
   ULONG   ReceiveToutCtr;
   ULONG   ModemStatusRegister;
   LONG    compareErrors;
   LONG    overrunSoftware;
   LONG    overrunHardware;
   LONG    NumberofTransfers;
   WORD    RcvErrorCount;
   WORD    rxTrigger;
   WORD    txTrigger;
   WORD    rxFlowOffLevel;
   WORD    rxFlowOnLevel;
   WORD    rxferCount;
   WORD    txferCount;
   BYTE    rxTimeout;
   BYTE    roundRobinIndex;
   BYTE    breakReceived;
   BYTE    rxHolding;
   BYTE    svcMask;
   BYTE    PortState;
   BYTE    PortType;               /* master = 1 slave = 0  */
   BYTE    SetVector;
   BYTE    dmaChannel;
   BYTE    dmaTime;
   BYTE    UartPrescaler;
   BYTE    Error1;
   BYTE    Error2;
   BYTE    bitRate;
   BYTE    dataBits;
   BYTE    stopBits;
   BYTE    parityMode;
   BYTE    flowCtrlMode;
   BYTE    flowCtrlEsp1;
   BYTE    flowCtrlEsp2;
   BYTE    breakState;
   BYTE    transmitXON;
   BYTE    transmitXOFF;
   BYTE    RxMode;
   BYTE    TxMode;
   BYTE    ReceiveMoveType;
   BYTE    TransmitMoveType;
   WORD    Sid;
   WORD    SidAccumlative;
   WORD    externalControl;
   WORD    externalStatus;
   char    *rcvbfrStart;
   char    *rcvbfrEnd;    
   char    *rcvbfrDrvr;
   char    *rcvbfrPC;    
   WORD    xmtbfrSize;
   WORD    rcvbfrSize;    
   struct  IRQS *IRQValues;
};

/***********************************************************************/
/*     Port configuration table structure.                             */
/***********************************************************************/
struct   PortConfiguration
{
   char    *String;
   char    *Format;
};

/***********************************************************************/
/* IRQ structure.                                                      */
/***********************************************************************/
struct   IRQS
{
   char    *CfgIRQ;
   BYTE    ESPIrq;
   BYTE    Vector;
   BYTE    PIC;
   BYTE    MASK;
   BYTE    DmaIrq;
   BYTE    MaskRegisterBit;
};
#pragma pack()

/***********************************************************************/
/*     ESP command block structure.                                    */
/***********************************************************************/
#pragma pack(1)
typedef struct 
{
   WORD     ioBase;
   BYTE     command;
   BYTE     writeParms[ 5 ];
   union    {
               BYTE  parms[ 2 ];
               WORD  val;
            } uRead;
   BYTE     IntFlagAction;    // either DONT_CARE or LEAVE_INTS_ALONE
} ESPCMDBLOCK;
#pragma pack()

/***********************************************************************/
/* Misc structures.                                                    */
/***********************************************************************/
struct DATABITS
{  
   BYTE    Value;
   BYTE    DataBits;
};

struct WordWord
{  
   WORD    Word1;
   WORD    Word2;
};

struct PARITY
{  
   char    *String;
   BYTE    Parity;
};

struct STOPBITS
{  
   char    *String;
   BYTE    StopBits;
};


/***********************************************************************/
/* Values for EspBoard Flag                                            */
/***********************************************************************/

#define FX_USE_PIT      0x0001
#define FX_ESP_BUSY     0x0002
#define FX_DMA_READ     0x0004
#define FX_RESET_MASK   0x0008
#define FX_ON_SLAVE_PIC 0x0010
#define FX_BUS_AT       0x0020
#define FX_TRANSMIT_ON  0x0040
#define FX_RECEIVE_ON   0x0080
#define DONT_CARE          0     
#define LEAVE_INTS_ALONE   1
#define SET( value, mask )    value |= mask
#define CLR( value, mask )    value &= (~mask)

/***********************************************************************/
/* ESP register offsets                                                */
/***********************************************************************/
#define ESP_R_RDY       0
#define ESP_R_SID       1
#define ESP_R_RX1       2
#define ESP_R_TX1       2
#define ESP_R_RX2       3
#define ESP_R_TX2       3
#define ESP_R_STATUS1   4
#define ESP_R_CMD1      4
#define ESP_R_STATUS2   5
#define ESP_R_CMD2      5
#define ESP_R_DMA_RX    6
#define ESP_R_DMA_TX    6

/***********************************************************************/
/* Ready Register bit masks                                            */
/***********************************************************************/
#define ESP_RDY_RX1     0x01
#define ESP_RDY_TX1     0x02
#define ESP_RDY_RX2     0x04
#define ESP_RDY_TX2     0x08
#define ESP_RDY_STATUS1 0x10              
#define ESP_RDY_CMD1    0x20                   
#define ESP_RDY_STATUS2 0x40                     
#define ESP_RDY_CMD2    0x80             

/***********************************************************************/
/* Service ID bit masks                                                */
/***********************************************************************/
#define ESP_SID_RX1     0x01
#define ESP_SID_TX1     0x02
#define ESP_SID_ERROR1  0x04
#define ESP_SID_DMAFIN  0x08
#define ESP_SID_RX2     0x10
#define ESP_SID_TX2     0x20
#define ESP_SID_ERROR2  0x40
#define ESP_SID_DMATO   0x80
#define ESP_SID_RXBOTH  0x11
#define ESP_SID_TXBOTH  0x22
#define ESP_SID_TXRXBOTH 0x33
#define ESP_SID_PORT1   0x07
#define ESP_SID_PORT2   0x70

/***********************************************************************/
/* Error Status bit masks                                              */
/***********************************************************************/
#define ESP_ERR1_RXTO      0x01              
#define ESP_ERR1_RXOF      0x02        
#define ESP_ERR1_PARITY    0x04          
#define ESP_ERR1_FRAME     0x08        
#define ESP_ERR1_BRK       0x10        
#define ESP_ERR1_MSR       0x20           
#define ESP_ERR1_FLOWTO    0x40        
#define ESP_ERR1_BRKFIN    0x80          
#define ESP_ERR2_OFFREMOTE 0x01     
#define ESP_ERR2_OFFLOCAL  0x02         
#define ESP_ERR2_ONLOCAL   0x04
#define ESP_ERR2_ONREMOTE  0x08            // oh oh same as PIT
#define ESP_ERR2_PIT       0x08

/***********************************************************************/
/* Bit masks to build parameters for Set FCType command                */
/***********************************************************************/
#define ESP_FC1_RX_XO      0x01
#define ESP_FC1_XPARENT_XO 0x02
#define ESP_FC1_RX_RTS     0x04
#define ESP_FC1_RX_DTR     0x08
#define ESP_FC1_ON_RTS     0x10      
#define ESP_FC1_ON_DTR     0x20     
#define ESP_FC1_LOOP       0x40       
#define ESP_FC1_TX_XO      0x80        
#define ESP_XO_FLOW_CONTROL 0x83
#define ESP_FC2_TX_CTS     0x10     
#define ESP_FC2_TX_DSR     0x20            

#define ESP_NUM_CMDS       0x20

/***********************************************************************/
/* Esp command definitions.                                            */
/***********************************************************************/
#define ESP_CMD_RESET         0x00
#define ESP_CMD_GETTEST       0x01
#define ESP_CMD_GDIPS         0x02
#define ESP_CMD_DLOAD         0x03
#define ESP_CMD_SETIRQ        0x04
#define ESP_CMD_SETDMATIME    0x05
#define ESP_CMD_SETSVCMASK    0x06
#define ESP_CMD_SETERRMASK    0x07
#define ESP_CMD_SETFCTYPE     0x08
#define ESP_CMD_SETFCCHAR     0x09
#define ESP_CMD_SETFCLEVEL    0x0A
#define ESP_CMD_SETTRIGGER    0x0B
#define ESP_CMD_SETRXTIME     0x0C
#define ESP_CMD_SETFCTIME     0x0D
#define ESP_CMD_WRITEUART     0x0E
#define ESP_CMD_READUART      0x0F
#define ESP_CMD_SETMODE       0x10
#define ESP_CMD_CLEARDMAREQ   0x11
#define ESP_CMD_GETERRS       0x12
#define ESP_CMD_GETUARTS      0x13
#define ESP_CMD_GETRXBYTES    0x14
#define ESP_CMD_GETTXSPACE    0x15
#define ESP_CMD_DMARX         0x16
#define ESP_CMD_DMATX         0x17
#define ESP_CMD_XOFFLOCAL     0x18
#define ESP_CMD_XONLOCAL      0x19
#define ESP_CMD_BREAK         0x1A
#define ESP_CMD_FLUSHRX       0x1B
#define ESP_CMD_FLUSHTX       0x1C
#define ESP_CMD_SETBAUD       0x1D
#define ESP_CMD_GETMODE       0x1E
#define ESP_CMD_SETPIT        0x1F
#define ESP_CMD_ENHANCED_IRQ  0x1F    // P-38
#define ESP_CMD_RIP           0x20
#define ESP_CMD_GETSIDMASK    0x21
#define ESP_CMD_GETRCVERRCNT  0x22
#define ESP_CMD_SETUARTCLOCK  0x23


/***********************************************************************/
/* definition defaults.                                                */
/***********************************************************************/
#define ESP_DEF_TRIGGER_TX    512      
#define ESP_DEF_RXFLOW_OFF    768
#define ESP_DEF_RXFLOW_ON     512
#define ESP_DEF_REIRQ_TOUT    250
#define EMPTY_XMIT_FIFO       0x3FF 
/***********************************************************************/
/*     Modem status values.                                            */
/***********************************************************************/
#define ESP_MDMSTAT_CTS        (0x10)
#define ESP_MDMSTAT_DSR        (0x20)
#define ESP_MDMSTAT_RI         (0x40)
#define ESP_MDMSTAT_DCD        (0x80)


/***********************************************************************/
/*    Control and data ports of 8250 UART                              */
/***********************************************************************/
#define UART_RCVDATA    (0)
#define UART_XMTDATA    (0)
#define UART_INTENB     (1)
#define UART_INTID      (2)
#define UART_LINECTRL   (3)
#define UART_MDMCTRL    (4)
#define UART_LINESTAT   (5)
#define UART_MDMSTAT    (6)
#define UART_SCRATCH    (7)
#define UART_LINECTRL_DATALEN   (0x03)
#define UART_LINECTRL_STOPBITS  (0x04)
#define UART_LINECTRL_PARITYEN  (0x08)
#define UART_LINECTRL_EPARITY   (0x10)
#define UART_LINECTRL_FPARITY   (0x20)
#define UART_LINECTRL_PARITYMSK (0x38)
#define UART_LINECTRL_SETBREAK  (0x40)
#define UART_LINECTRL_DLAB      (0x80)


#define LOBYTE(w)	((BYTE)(w))
#define HIBYTE(w)	((BYTE)(((WORD)(w) >> 8) & 0xff))

/********************************************  *******************/
/* DMA controller addresses and commands                               */
/***********************************************************************/
#define DMA_type_write  0x04
#define DMA_type_read   0x08
#define Transfer_Data	0x04
#define Write_Mem	      0x08
#define DMA_MODE        0x0B
#define DMA_MODE_READ   0x08
#define DMA_MODE_WRITE  0x04
#define DMA_MASK        0x0A
#define DMA_CLEAR       0x0C
#define DMA_ADDRESS     0x00
#define DMA_PAGE_1      0x83
#define DMA_PAGE_3      0x82

/***********************************************************************/
/*  PS/2 DMA specific ports                                         `  
/***********************************************************************/
#define DMA_XFN 	      0x18
#define DMA_EXE 	      0x1A

/***********************************************************************/
/* PS/2 extended commands                                              */
/***********************************************************************/
#define Set_IO_Adr		0x00
#define Set_Mem_Adr		0x20
#define Get_Mem_Adr		0x30
#define Set_Count		   0x40
#define Get_Count		   0x50
#define Get_Status		0x60
#define Set_Mode		   0x70
#define Set_Arbus		   0x80
#define Set_Chn_Mask		0x90
#define Reset_Chn_Mask	0xA0
#define Master_Clear	   0xD0


typedef struct 
{
   BYTE  numWrites;
   BYTE  numReads;
} CMDACTION;

#define    ESP_NUM_P38_CMDS    4           // { num_writes, num_reads }
   

/**************************** EXTERNAL VARIABLES **************************/
extern   BYTE  computer;
extern   char   *configfile;
extern   FILE   *file;
extern   double   TotalTXbytes[];
extern   double   TotalRXbytes[];
extern   BYTE      CompareStartCharacter[];  
extern   int      block_size[];
//extern   BYTE     rx_buffer[][MAX_COUNT];
extern   BYTE     rx_buffer[][MAX_RECV_COUNT];
extern   BYTE     tx_buffer[][MAX_COUNT];
extern   BYTE     save_tx_buffer[MAX_COUNT];
extern   char     *testfile;
extern   int      result;
extern   int      byte_count;
extern   int      i;   
extern   int      key;
extern   int      quit;
extern   BYTE     EspAdapterType;
extern   char     ConfigFileStr[];
extern   int      ServiceID;
extern   CMDACTION EspCmdActions[];    
extern   BYTE      Esp1PIC;
extern   BYTE      Esp2PIC;   
extern   struct PortConfiguration PortConfig[];
extern   struct ESP38 P38Ports[];
extern   struct IRQS  ESP2IRQ [];
extern   LONG BaudRateTable[];
extern   WORD MemoryMapAddressAT[];
extern   WORD MemoryMapAddressMCA[];
extern   WORD EnhancedAddressAT[];
extern   WORD EnhancedAddressMCA[];
extern   WORD BitRateTable[];
extern   struct PARITY ParityTable[];
extern   struct STOPBITS StopBitsTable[];
extern   struct DATABITS DataBitsTable[];
extern   BYTE     NumberOfPorts;  


/************************** FUNCTION DECLARATIONS *************************/
void  set_cursor_pos( unsigned char,  unsigned char );
int   getconfig(void);
int   getpattern(void);
void  cls(void);
int   FindESPPorts(int);
void  InitVariables(void);
void  SetUpHardwareHandler(BYTE);
void  ESPIsr(WORD);
void  ESPInterrupt1(void);
void  ESPInterrupt2(void);
void  TransmitPacket(BYTE,BYTE);
void  ResetIRQVector(void);
void  UpdateStatus(BYTE, BYTE);
void  clsPart(BYTE, BYTE, BYTE, BYTE, BYTE);
void  DisplayTransmitPacket(BYTE, char *, BYTE, ULONG *, BYTE);
void  DisplayReceivePacket(BYTE, BYTE, BYTE, ULONG *, BYTE);
void  WakeUpBoard(void);
void  ReceivePacket(BYTE,BYTE);
void  ReceiveData (BYTE);
void  IssueEspCmd( ESPCMDBLOCK *);
void  IssueEspCmdLeaveIntsAlone( ESPCMDBLOCK *);
void  TheRealIssue( ESPCMDBLOCK * );
void  TheRealIssueNoWait( ESPCMDBLOCK * );
void  DmaTxComplete( WORD );
void  DmaRxComplete( WORD );
void  ErrorESP( WORD );
WORD  ReadTransferCount( WORD);
void  RxService(WORD , BYTE);      
void  TxService(WORD , BYTE);      
void  ProgramDmaController( WORD, char *, WORD );
void  DoEspConfig(void);
void  InitializePort(WORD );
void  ResetPortParameters( WORD );
void  ResetIORegsParameters( WORD Port );
void  DisplayIt(char *, WORD, BYTE, BYTE);
void  DumpReceiveData(BYTE);                              
void  PortDataConfigure(WORD);
int   ConfigurePort ( WORD );
void  DevelopExternalStatus( WORD );
void  DoMemoryMapXfer(WORD, char *, WORD, BYTE);
void  DoProgramIOXfer(WORD, char *, WORD, BYTE);
void  DisplayConfiguration(void);
void  DisplayConfigData(WORD);
WORD  DoReceiveFilter(WORD, char *, WORD, WORD);
void   TransmitandCompare(BYTE);
void   ReceiveandCompare(BYTE);
void   CompareReceivePacket(BYTE , ULONG *);
void   CompareIt(char *, WORD, BYTE);
void   SetServiceMask (BYTE , BYTE);
void   SendRemoteCharacter(BYTE , BYTE );
void   DisplayLowLevelStatus(void);










