/**************************************************************************
 * ESPISR.C                                                               *
 * Contains Interrupt Service Routine for ESP test.                       *
 **************************************************************************/
/*** NOTICE *** NOTICE *** NOTICE *** NOTICE *** NOTICE *** NOTICE ***
 ***                                                               ***
DISCLAIMER: THE PRODUCT IS PROVIDED TO DEVELOPER "AS IS" WITHOUT WARRANTY
OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE 
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE,
AND THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PRODUCT IS 
ASSUMED BY DEVELOPER, INCLUDING ALL COSTS OF SERVICING, REPAIRS OR CORRECTIONS.  
DEVELOPER AGREES THAT HAYES WILL NOT BE LIABLE FOR ANY DAMAGES DIRECTLY OR 
INDIRECTLY RELATED TO THE USE OF THE PRODUCT, INCLUDING BUT NOT LIMITED
TO, LOST PROFITS, INDIRECT, SPECIAL OR CONSEQUENTIAL DAMAGES, WHETHER BASED
ON CONTRACT, TORT OR ANY OTHER LEGAL THEORY.  DEVELOPER ACKNOWLEDGES THAT 
THE PRODUCT IS STILL UNDER DEVELOPMENT AND THAT HAYES DOES NOT WARRANT 
THAT THE PRODUCT IN ITS PRESENT OR FUTURE FORM WILL MEET DEVELOPER'S NEEDS.
 ***                                                               ***
 *** NOTICE *** NOTICE *** NOTICE *** NOTICE *** NOTICE *** NOTICE ***/

/***********************************************************************/
/*    Copyright 1993 Hayes Microcomputer Products, Inc.                */
/*    All rights reserved.                                             */
/*    Use of copyright notice does not imply publication.              */
/***********************************************************************/
/***********************************************************************/
/*                                                                     */
/*                                                                     */
/*     NOTE:   This code is specifically for the ESP2 board.           */
/*                                                                     */
/*                     T E S T    C O D E                              */
/*                                                                     */
/*     This code uses the Hayes Enhanced Serial Interface (ESI) Ver 2  */     
/*     to operate the ESP2 adapter.  Please refer to that manual for   */
/*     more detail on the commands.                                    */
/***********************************************************************/

/****************************** INCLUDES *******************************/
#include  <conio.h>
#include  <stdio.h>
#include  <io.h>
#include  <dos.h>
#include  "esptype.h"
#include  "espfunc.h"


/***********************************************************************/
/*     ESP command table along with the number of DATA bytes associated*/
/*     with the command.                                               */
/***********************************************************************/
CMDACTION EspCmdActions[ ESP_NUM_CMDS + ESP_NUM_P38_CMDS] =    
   {
      { 0, 1 }                            // ESP_CMD_RESET
      ,{ 0, 2 }                           // ESP_CMD_GETTEST
      ,{ 0, 1 }                           // ESP_CMD_GDIPS
      ,{ 0, 0 }                           // ESP_DLOAD not handled
      ,{ 1, 0 }                           // ESP_CMD_SETIRQ
      ,{ 1, 0 }                           // ESP_CMD_SETDMA
      ,{ 1, 0 }                           // ESP_CMD_SETSVCMASK
      ,{ 2, 0 }                           // ESP_CMD_SETERRMASK
      ,{ 2, 0 }                           // ESP_CMD_SETFCTYPE
      ,{ 5, 0 }                           // ESP_CMD_SETFCCHARS
      ,{ 4, 0 }                           // ESP_CMD_SETFCLEVEL
      ,{ 4, 0 }                           // ESP_CMD_SETTRIGGER
      ,{ 1, 0 }                           // ESP_CMD_SETRXTIME
      ,{ 1, 0 }                           // ESP_CMD_SETFCTIME
      ,{ 2, 0 }                           // ESP_WRITE_UART
      ,{ 1, 1 }                           // ESP_READ_UART
      ,{ 1, 0 }                           // ESP_CMD_SETMODE
      ,{ 0, 0 }                           // ESP_CMD_CLEARDMAREQ
      ,{ 0, 2 }                           // ESP_GETERRS
      ,{ 0, 2 }                           // ESP_GETUARTS
      ,{ 0, 2 }                           // ESP_GETRXBYTES
      ,{ 0, 2 }                           // ESP_GETTXSPACE
      ,{ 0, 1 }                           // ESP_CMD_DMARX
      ,{ 0, 1 }                           // ESP_CMD_DMATX
      ,{ 0, 0 }                           // ESP_CMD_XOFFLOCAL
      ,{ 0, 0 }                           // ESP_CMD_XONLOCAL
      ,{ 1, 0 }                           // ESP_CMD_BREAK
      ,{ 0, 0 }                           // ESP_CMD_FLUSHRX
      ,{ 0, 0 }                           // ESP_CMD_FLUSHTX
      ,{ 2, 0 }                           // ESP_CMD_SETBAUD
      ,{ 0, 2 }                           // ESP_CMD_GETMODE
      ,{ 1, 0 }                           // ESP_CMD_SETPIT (1Fh) or 
                                          // ESP_CMD_ENHANCED_IRQ (P-38)
      ,{ 1, 0 }                           // ESP_CMD_RIP  (20h)
      ,{ 0, 1 }                           // ESP_CMD_GETSIDMASK  (21h)
      ,{ 0, 2 }                           // ESP_CMD_GETRCVERRCNT (22h)
      ,{ 1, 0 }                           // ESP_CMD_SETUARTCLOCK (23h)

   };

/***********************************************************************/
/*   Round robin mask and table to allow equal access to the ISR.      */
/***********************************************************************/
BYTE  RoundRobinMask[] = { 0x01,    // Rx1FIFO int
                           0x02 };  // Tx1FIFO int
#define NUM_RR_ENTRIES (sizeof(RoundRobinMask) / sizeof(RoundRobinMask[ 0 ]))
typedef void (*PSERVICE)( WORD, BYTE );
PSERVICE RoundRobinService[] =   {  RxService,
                                    TxService};

/***********************************************************************/
/* Entry into the ISR comes from the ESPENTRY.ASM routines.  We have   */
/* entry for up to two IRQ's.  Each entry set's up DS and ES and sets  */
/* up it's own stack.  The EOI of the interrupt does not happen here,  */
/* it is done ESPENTRY.ASM as the very last thing.                     */
/***********************************************************************/

/* ==========================ESPIsr===========================
 * Interrupt Routine for ESP
 * Return: none
 */
void   ESPIsr(WORD Port)
{
   int i, index;

   ESPCMDBLOCK cmdBlock;


   _asm int 3

/***********************************************************************/
/*  Get Service Bit Id.  This will tells us why we are here.           */
/***********************************************************************/
   P38Ports[Port].Sid =
          inp ((WORD) (P38Ports[Port].EnhancedAddress + 
                           (WORD) ESP_CMD_SID));


   P38Ports[Port].ISREntry++;      // debug status only
/***********************************************************************/
/* Accumlate our SID so the ESPTEST.C can report the entire SID.       */
/***********************************************************************/
   P38Ports[Port].SidAccumlative |= P38Ports[Port].Sid;

/***********************************************************************/
/* Set the receive timeout to zero to prevent reentrancy for slower bus*/
/* machines.  4-13-93                                                  */    
/***********************************************************************/
/***********************************************************************/
/* 4-26-93  We should not be doing this unless Xon/Xoff flow control   */
/***********************************************************************/
   if ((P38Ports[Port].flowCtrlEsp1 & ESP_XO_FLOW_CONTROL))
   {
       cmdBlock.command = ESP_CMD_SETRXTIME;
       cmdBlock.writeParms[0] = 0;
       cmdBlock.ioBase = P38Ports[Port].EnhancedAddress;
       IssueEspCmdLeaveIntsAlone( &cmdBlock );
       cmdBlock.command = ESP_CMD_SETRXTIME;
       cmdBlock.writeParms[0] = P38Ports[Port].rxTimeout;
       cmdBlock.ioBase = P38Ports[Port].EnhancedAddress;
       IssueEspCmdLeaveIntsAlone( &cmdBlock );
       ErrorESP( Port );
   }
   else  if (P38Ports[Port].Sid & ESP_SID_ERROR1) 
           ErrorESP( Port );

/***********************************************************************/
/* This indicates that upon exit we should reset out SID mask.         */
/***********************************************************************/
   SET(P38Ports[Port].flags, FX_RESET_MASK );

/***********************************************************************/
/* Handle DMA Finished                                                 */
/***********************************************************************/
   if (P38Ports[Port].Sid & ESP_SID_DMAFIN)
   {
/***********************************************************************/
/* See what type of DMA.  This is a transmit.                          */
/***********************************************************************/
       if (P38Ports[Port].flags & FX_DMA_READ)     // read from memory
           DmaTxComplete(Port);
       else
/***********************************************************************/
/* This is a receive DMA.                                              */
/***********************************************************************/
           DmaRxComplete(Port);
   }

/***********************************************************************/
/* Handle RX and TX fifo                                               */
/* Process any Rx or Tx ints in round-robin order.                     */
/***********************************************************************/
   for (i = NUM_RR_ENTRIES; i > 0; i--)
   {
      /* Compare current sid value to "next sid to process".   */
      index = P38Ports[Port].roundRobinIndex;
      if (P38Ports[Port].Sid & (RoundRobinMask[ index ]))
      {
         (*(RoundRobinService[ index ]))(Port, FALSE );
      }
      // Increment "next sid to process", accounting for wrap.
      if (++P38Ports[Port].roundRobinIndex >= NUM_RR_ENTRIES)
         P38Ports[Port].roundRobinIndex = 0;

/***********************************************************************/
/* Check to see if one of the subroutines have set ESP busy.           */
/***********************************************************************/
      if (P38Ports[Port].flags & FX_ESP_BUSY)
      {
         break;
      }
   
   }

   if (P38Ports[Port].flags & FX_RESET_MASK)
   {
/***********************************************************************/
/* Reading the SID at this point will restart the reinterruptable timer*/
/***********************************************************************/
       inp ((WORD) (P38Ports[Port].EnhancedAddress + (WORD) ESP_CMD_SID));
#ifdef DONOTDO
       cmdBlock.command = ESP_CMD_SETSVCMASK;
       cmdBlock.ioBase  = P38Ports[Port].EnhancedAddress;
       cmdBlock.writeParms[0] = 0;
       IssueEspCmdLeaveIntsAlone( &cmdBlock ); 

       cmdBlock.command = ESP_CMD_SETSVCMASK;
       cmdBlock.ioBase  = P38Ports[Port].EnhancedAddress;
       cmdBlock.writeParms[0] = P38Ports[Port].svcMask;
       IssueEspCmdLeaveIntsAlone( &cmdBlock );
#endif

   }

   return;

}

/***********************************************************************/
/*                                                                     */
/*                                                                     */
/*     DmaTXComplete                                                   */
/*                                                                     */
/*                                                                     */
/*     This handles a Dma Transmit complete interrupt.                 */
/*                                                                     */
/*                                                                     */
/*                                                                     */
/*                                                                     */
/***********************************************************************/
void DmaTxComplete( WORD Port )
{

/***********************************************************************/
/* Clear DmaStatus flag to indicate no DMA is processing.              */
/***********************************************************************/
   P38Ports[Port].DMAStatus = 0;

/***********************************************************************/
/* Update some statistics counter to be displayed by ESPTEST.C         */
/***********************************************************************/
   P38Ports[Port].transmitBytes += P38Ports[Port].txferCount;
   P38Ports[Port].NumberofTransfers++;

/***********************************************************************/
/* Clear the flag of ESP being busy and currently transmitting.        */
/***********************************************************************/
   CLR( P38Ports[Port].flags, (FX_ESP_BUSY|FX_TRANSMIT_ON));
   return;

}


/***********************************************************************/
/*                                                                     */
/* DmaRxComplete                                                       */
/*                                                                     */
/* Receive DMA complete interrupt.                                     */
/*                                                                     */
/*                                                                     */
/*                                                                     */
/*                                                                     */
/***********************************************************************/
void DmaRxComplete( WORD Port )
{

/***********************************************************************/
/* Update break received flag.                                         */
/***********************************************************************/
   if (P38Ports[Port].breakReceived)
      P38Ports[Port].breakReceived = FALSE;    

/***********************************************************************/
/* Update receive que ptr, accounting for wrap.                        */
/***********************************************************************/
   if ( (P38Ports[Port].rcvbfrDrvr += P38Ports[Port].rxferCount) > 
           P38Ports[Port].rcvbfrEnd)
       P38Ports[Port].rcvbfrDrvr = P38Ports[Port].rcvbfrStart;

/***********************************************************************/
/* Update DmaStatus flag to indicate no DMA activity.                  */
/***********************************************************************/
   P38Ports[Port].DMAStatus = 0;

/***********************************************************************/
/* Update counter statistics fields.                                   */
/***********************************************************************/
   P38Ports[Port].NumberofTransfers++;
   P38Ports[Port].receiveBytes += P38Ports[Port].rxferCount;

/***********************************************************************/
/* Set flag to indicate some receive data is available and clear the   */
/* the ESP busy bit.                                                   */
/***********************************************************************/
   SET (P38Ports[Port].flags, FX_RECEIVE_ON);
   CLR( P38Ports[Port].flags, FX_ESP_BUSY );
   return;

}


/***********************************************************************/
/*                                                                     */
/*    ErrorEsp                                                         */
/*                                                                     */
/*                                                                     */
/* Handle ESP error stuff.                                             */
/*                                                                     */
/*                                                                     */
/*                                                                     */
/*                                                                     */
/***********************************************************************/
void ErrorESP( WORD Port )
{        
   BYTE     err1, err2;
   WORD     bytesActuallyXferred;
   ESPCMDBLOCK cmdBlock;

/***********************************************************************/
/* Update debug stuff.                                                 */
/***********************************************************************/

/***********************************************************************/
/* If we get hear we are finished DMA so clear the ESP busy flag.      */
/***********************************************************************/
   CLR( P38Ports[Port].flags, FX_ESP_BUSY );

/***********************************************************************/
/* Read the error port to get error information.                       */
/***********************************************************************/
   cmdBlock.command = ESP_CMD_GETERRS ;   
   cmdBlock.ioBase = P38Ports[Port].EnhancedAddress;
   IssueEspCmdLeaveIntsAlone( &cmdBlock );
   P38Ports[Port].Error1 = err1 
       = cmdBlock.uRead.parms[1];     // STATUS1 = parms[1];
   P38Ports[Port].Error2 = err2 
       = cmdBlock.uRead.parms[0];     // STATUS2 = parms[0];

/***********************************************************************/
/* First check for break or line error                                 */
/***********************************************************************/
   if (err1 & (ESP_ERR1_BRK | ESP_ERR1_FRAME | ESP_ERR1_PARITY ))
   {
/***********************************************************************/
/* This error may have caused last DMA transfer to be aborted.         */
/* How many bytes were actually transferred?                           */
/***********************************************************************/
       if (P38Ports[Port].DMAStatus != 0)
       {   
           bytesActuallyXferred = P38Ports[Port].rcvbfrSize - ReadTransferCount( Port );

/***********************************************************************/
/* Update statistics counters.                                         */
/***********************************************************************/
           P38Ports[Port].receiveBytes += bytesActuallyXferred;

/***********************************************************************/
/* Update que ptrs and count.                                          */
/***********************************************************************/
           if ( (P38Ports[Port].rcvbfrDrvr += P38Ports[Port].rxferCount) > 
                                       P38Ports[Port].rcvbfrEnd)
               P38Ports[Port].rcvbfrDrvr = P38Ports[Port].rcvbfrStart;

/***********************************************************************/
/* Clear DMA status flag.                                              */
/***********************************************************************/
           P38Ports[Port].DMAStatus = 0;
       }

/***********************************************************************/
/* Update error statistics.                                            */
/***********************************************************************/
      if (err1 & ESP_ERR1_FRAME)
         P38Ports[Port].framingErrors++;
      if (err1 & ESP_ERR1_PARITY)
         P38Ports[Port].parityErrors++;

/***********************************************************************/
/* Set to TRUE here when break received, set to FALSE when next byte   */
/* of data is received.                                                */
/***********************************************************************/
      if (err1 & ESP_ERR1_BRK)
      {  
           P38Ports[Port].breakReceived = TRUE;    
      }
   }

/***********************************************************************/
/* Next check for incoming data --  RxTimeout                          */
/***********************************************************************/
   if (err1 & ESP_ERR1_RXTO) 
   {
       P38Ports[Port].ReceiveToutCtr++;
/***********************************************************************/
/* If no outstanding DMA transfers, call RxService to grab incoming    */
/* data. Set useItOrLoseIt parameter to TRUE -- if we don't grab       */
/* it now, ESP won't notify us again until more comes in.              */
/* If outstanding transfer, we're just hosed because can't transfer    */
/* now but ESP won't notify us again until more arrives.               */
/***********************************************************************/
       if (P38Ports[Port].DMAStatus == 0)
           RxService(Port, TRUE );      
   }

/***********************************************************************/
/* If P38 and the Onremote bit is set then send an XON to local        */
/* talker.  However this is only need for Xon/Xoff and transparent     */
/***********************************************************************/

   if (P38Ports[Port].flowCtrlEsp1 & ESP_XO_FLOW_CONTROL)
   {
       if ( err2 & ESP_ERR2_ONREMOTE)
       {
           cmdBlock.ioBase = P38Ports[Port].EnhancedAddress;
           cmdBlock.command = ESP_CMD_XOFFLOCAL;     
           IssueEspCmdLeaveIntsAlone( &cmdBlock );
            
           cmdBlock.command = (BYTE) (ESP_CMD_GETTXSPACE ); 
           IssueEspCmdLeaveIntsAlone( &cmdBlock );
           if (cmdBlock.uRead.val == EMPTY_XMIT_FIFO)       
           {
   /***********************************************************************/
   /*     Flush our transmit queue because it is empty.                   */
   /***********************************************************************/
               cmdBlock.command = ESP_CMD_FLUSHTX;     
               IssueEspCmdLeaveIntsAlone( &cmdBlock );
           }
           else
           {
   /***********************************************************************/
   /* try to do at a later time.  We need set the Transmit FIFO trigger   */
   /* to be completely empty when we receive a TX interrupt.  This will   */
   /* allow us to flush the fifo and thus reset the FCSM.  Also, turn on  */
   /* the TX SID bit if not already set.                                  */ 
   /***********************************************************************/
               P38Ports[Port].flowCtrlMode |= ESP_ERR2_ONREMOTE;           
               cmdBlock.command = ESP_CMD_SETTRIGGER ;
               cmdBlock.writeParms[0] = 
                   HIBYTE(P38Ports[Port].rxTrigger);  // first: RX trig MSB
               cmdBlock.writeParms[1] = 
                   LOBYTE(P38Ports[Port].rxTrigger);  // next:  RX trig LSB
               cmdBlock.writeParms[2] = 
                   HIBYTE(EMPTY_XMIT_FIFO);           // next:  TX trig MSB
               cmdBlock.writeParms[3] = 
                   LOBYTE(EMPTY_XMIT_FIFO);           // last:  TX trig LSB
               IssueEspCmdLeaveIntsAlone( &cmdBlock );
               if (!(P38Ports[Port].svcMask & ESP_SID_TX1))
               {
                   cmdBlock.writeParms[0] = P38Ports[Port].svcMask;
                   cmdBlock.writeParms[0] |= ESP_SID_TX1;
                   cmdBlock.command = ESP_CMD_SETSVCMASK;    
                   IssueEspCmdLeaveIntsAlone( &cmdBlock );
               }    
           }
           cmdBlock.command = ESP_CMD_XONLOCAL;     
           IssueEspCmdLeaveIntsAlone( &cmdBlock );
       }

/***********************************************************************/
/* If P38 and the Offremote bit is set then send an XOFF to local      */
/* talker.  However this is only needed for Xon/Xoff and transparent   */
/***********************************************************************/
       if ( err2 & ESP_ERR2_OFFREMOTE)
       {
           if (P38Ports[Port].flowCtrlMode & ESP_ERR2_ONREMOTE)
               P38Ports[Port].flowCtrlMode |= ESP_ERR2_OFFREMOTE;           
       }
   }    
/***********************************************************************/
/* Overrun in ESP buffers?                                             */
/***********************************************************************/
   if (err1 & ESP_ERR1_RXOF)
      P38Ports[Port].overrunSoftware++;

/***********************************************************************/
/* MSR change                                                          */
/*     If Modem Status Register has changed then you might want to     */
/*     read the modem status register to get change.                   */
/***********************************************************************/
   if (err1 & ESP_ERR1_MSR)
   {
         P38Ports[Port].ModemStatusRegister++;
   }

 
}


/***********************************************************************/
/*                                                                     */
/*     RxService                                                       */
/*                                                                     */
/*                                                                     */
/*                                                                     */
/* Receiver has data for us to process.                                */
/*                                                                     */
/*                                                                     */
/*                                                                     */
/*                                                                     */
/*                                                                     */
/*                                                                     */
/***********************************************************************/
void RxService(WORD Port, BYTE useItOrLoseIt )
{
   LONG        spaceAvail;
   char        *pDrvr, *pPC;
   ESPCMDBLOCK cmdBlock;


/***********************************************************************/
/* Calculate how much space available in receive queue.                */
/***********************************************************************/
   pDrvr = P38Ports[Port].rcvbfrDrvr;
   pPC =   P38Ports[Port].rcvbfrPC;
   if (pPC > pDrvr)       
   {
      spaceAvail = pPC - pDrvr - 1; 
   }
   else
   {
/***********************************************************************/
/* Buffer free space is in two chunks: from Drvr to End, and from      */
/* Start to (PC - 1). DMA must be contiguous, so do only the first     */
/* chunk now.                                                          */
/***********************************************************************/
      spaceAvail = P38Ports[Port].rcvbfrEnd - pDrvr + 1;
      if (pPC == P38Ports[Port].rcvbfrStart)
         spaceAvail--;  
   }

   if (spaceAvail)
   {
/***********************************************************************/
/* Ask ESP how many bytes are waiting in rx buffer.                    */
/* Due to hardware bug we need to issue and ESI command before getting */
/* the number of receive bytes in the receive queue.  This is only a   */
/* problem if we issue the GETRXBYTES two times in a row.              */
/***********************************************************************/
       cmdBlock.ioBase = P38Ports[Port].EnhancedAddress;
       cmdBlock.command = (BYTE) (0xFF); 
       IssueEspCmdLeaveIntsAlone( &cmdBlock );
       cmdBlock.command = ESP_CMD_GETRXBYTES;   
       IssueEspCmdLeaveIntsAlone( &cmdBlock );

/***********************************************************************/
/* Paranoia check: ESP may not have any receive data!! Sometimes       */  
/* ESP interrupts for error, error byte says RXTIMEOUT, but when       */
/* we arrive here, ESP says no new data. Check for this condition,     */    
/* and get out of here -- whatever you do, don't start a DMA transfer  */
/* for 0 bytes, for it will never complete.                            */
/***********************************************************************/
       if (cmdBlock.uRead.val == 0)
           return;

/***********************************************************************/
/* Compare bytes available with space available to determine transfer  */
/* count.                                                              */
/***********************************************************************/
       P38Ports[Port].rxferCount = 
                   (WORD) ((cmdBlock.uRead.val < (WORD) spaceAvail) ? 
                            cmdBlock.uRead.val : (WORD) spaceAvail);

/***********************************************************************/
/* Determine type of transfer.                                         */
/***********************************************************************/
       if (P38Ports[Port].RxMode == DMA_RX)
       {
/***********************************************************************/
/*     Do a DMA transfer. Need to program the DMA controller.          */
/***********************************************************************/
           P38Ports[Port].DMAStatus = DMAWRITE;
           CLR( P38Ports[Port].flags, FX_DMA_READ );                              
           ProgramDmaController( Port, 
               P38Ports[Port].rcvbfrDrvr, P38Ports[Port].rxferCount );
/***********************************************************************/
/* Now that controller is ready, tell ESP to start transfer.           */
/***********************************************************************/
           cmdBlock.command = ESP_CMD_DMARX;
           IssueEspCmdLeaveIntsAlone( &cmdBlock );
           CLR( P38Ports[Port].flags, FX_RESET_MASK );
/***********************************************************************/
/* Clear ALL Tx and Rx bits in the Sid. This allows only one DMA       */
/* transfer per isr.  Tell everyone that we are ESP busy.              */
/***********************************************************************/
           CLR( P38Ports[Port].Sid, ESP_SID_TXRXBOTH );
           SET( P38Ports[Port].flags, FX_ESP_BUSY );
       }
       else    
       {
   /***********************************************************************/
   /* Read the receive error count to see if we have any errors.          */
   /***********************************************************************/
           cmdBlock.command = ESP_CMD_GETRCVERRCNT;   
           IssueEspCmdLeaveIntsAlone( &cmdBlock );
           if ((cmdBlock.uRead.val) ||             // we have some errors 
              (P38Ports[Port].RcvErrorCount))      // so filter the data.
           {
               P38Ports[Port].RcvErrorCount = 
                           DoReceiveFilter( Port, 
                                   P38Ports[Port].rcvbfrDrvr, 
                                   P38Ports[Port].rxferCount, 
                                   (cmdBlock.uRead.val +
                                   P38Ports[Port].RcvErrorCount));
           } 
           else
           {
               if (P38Ports[Port].RxMode == MEMORY_MAPPED)
/***********************************************************************/
/*     It looks like Memory mapped.                                    */
/***********************************************************************/
               {
                   DoMemoryMapXfer(Port, P38Ports[Port].rcvbfrDrvr, 
                           P38Ports[Port].rxferCount, FX_RECEIVE_ON);
               }
               else
/***********************************************************************/
/* It's programmed IO.                                                 */
/***********************************************************************/
               {
                   DoProgramIOXfer(Port, P38Ports[Port].rcvbfrDrvr, 
                           P38Ports[Port].rxferCount, FX_RECEIVE_ON);
               }
           }
       }
/***********************************************************************/
/*  Update pointers and statistics type fields for Memory mapped       */  
/*  and programmed I/O now because we do not get a finish              */
/*  indicator.                                                         */  
/***********************************************************************/
       if ((P38Ports[Port].RxMode == MEMORY_MAPPED) ||
           (P38Ports[Port].RxMode == CHAR_RX))
       {
           P38Ports[Port].receiveBytes += P38Ports[Port].rxferCount;
           if (P38Ports[Port].breakReceived)
               P38Ports[Port].breakReceived = FALSE;    
           if ( (P38Ports[Port].rcvbfrDrvr += P38Ports[Port].rxferCount) > 
               P38Ports[Port].rcvbfrEnd)
               P38Ports[Port].rcvbfrDrvr = P38Ports[Port].rcvbfrStart;
           SET (P38Ports[Port].flags, FX_RECEIVE_ON);
           CLR( P38Ports[Port].flags, FX_ESP_BUSY );
           P38Ports[Port].NumberofTransfers++;
       }

   }

   else
   {
/***********************************************************************/
/* Disable the RxFIFO interrupt and let ESP buffer up incoming data.   */
/* NOTE: The rx queue consumer thread is responsible for renabling     */
/* this interrupt when the queue is no longer full.                    */
/***********************************************************************/
       SET (P38Ports[Port].flags, FX_RECEIVE_ON);
       P38Ports[Port].rxHolding = TRUE;
       CLR( P38Ports[Port].svcMask, ESP_SID_RX1 );
       cmdBlock.command = ESP_CMD_SETSVCMASK;    
       cmdBlock.ioBase = P38Ports[Port].EnhancedAddress;
       cmdBlock.writeParms[0] = P38Ports[Port].svcMask;
       IssueEspCmdLeaveIntsAlone( &cmdBlock );
   }
   return;
}
 

/***********************************************************************/
/*                                                                     */
/*     TxService                                                       */
/*                                                                     */
/*                                                                     */
/* We got room in the transmitter so let's see if we have any work.    */
/*                                                                     */
/*                                                                     */
/*                                                                     */
/*                                                                     */
/*                                                                     */
/***********************************************************************/
void TxService( WORD Port, BYTE Nothing )
{
   signed long bytesToSend;
   ESPCMDBLOCK cmdBlock;


/***********************************************************************/
/* See if we have to do a kluge Xon/Xoff stuff.                                      */
/***********************************************************************/
   if (P38Ports[Port].flowCtrlMode & ESP_ERR2_ONREMOTE)
   {
       cmdBlock.ioBase = P38Ports[Port].EnhancedAddress;
   /***********************************************************************/
   /*     Flush our transmit queue because it is empty.                   */
   /***********************************************************************/
       cmdBlock.command = ESP_CMD_FLUSHTX;     
       IssueEspCmdLeaveIntsAlone( &cmdBlock );
       cmdBlock.command = ESP_CMD_SETTRIGGER ;
       cmdBlock.writeParms[0] = 
           HIBYTE(P38Ports[Port].rxTrigger);  // first: RX trig MSB
       cmdBlock.writeParms[1] = 
           LOBYTE(P38Ports[Port].rxTrigger);  // next:  RX trig LSB
       cmdBlock.writeParms[2] = 
           (ESP_DEF_TRIGGER_TX / 256);        // next:  TX trig MSB
       cmdBlock.writeParms[3] = 
           (ESP_DEF_TRIGGER_TX % 256);        // last:  TX trig LSB
       IssueEspCmdLeaveIntsAlone( &cmdBlock );
       if (!(P38Ports[Port].svcMask & ESP_SID_TX1))
       {
           cmdBlock.writeParms[0] = P38Ports[Port].svcMask;
           cmdBlock.command = ESP_CMD_SETSVCMASK;    
           IssueEspCmdLeaveIntsAlone( &cmdBlock );
       }    
       if (P38Ports[Port].flowCtrlMode & ESP_ERR2_OFFREMOTE)
       {
           outp((WORD) (P38Ports[Port].EnhancedAddress + 2), ASCII_XOFF);
       }
       P38Ports[Port].flowCtrlMode = 0;
   }    

/***********************************************************************/
/*  we always try to send the MAX.                                     */
/***********************************************************************/
   bytesToSend = P38Ports[Port].xmtbfrSize;

/***********************************************************************/
/* Check to see if ESPTEST.C has set the transmit flag for us.         */
/***********************************************************************/
   if (!(P38Ports[Port].flags & FX_TRANSMIT_ON))
   {
/***********************************************************************/
/* No so let's get out after some house keeping.                       */
/***********************************************************************/
       CLR( P38Ports[Port].svcMask, ESP_SID_TX1 );
       CLR( P38Ports[Port].Sid, ESP_SID_TX1 );
       cmdBlock.command = ESP_CMD_SETSVCMASK;  
       cmdBlock.ioBase = P38Ports[Port].EnhancedAddress;
       cmdBlock.writeParms[0] = P38Ports[Port].svcMask;
       IssueEspCmdLeaveIntsAlone( &cmdBlock );
       cmdBlock.command = ESP_CMD_GDIPS;     
       IssueEspCmdLeaveIntsAlone( &cmdBlock );
 
       return;
   }


   if (bytesToSend == 0)
   {
/***********************************************************************/
/* Tx que is empty so disable TxFIFO interrupt. Non-interrupt code     */
/* will reenable when it puts more data in que.                        */
/***********************************************************************/
       CLR( P38Ports[Port].svcMask, ESP_SID_TX1 );
       CLR( P38Ports[Port].Sid, ESP_SID_TX1 );
       cmdBlock.command = ESP_CMD_SETSVCMASK;    
       cmdBlock.ioBase = P38Ports[Port].EnhancedAddress;
       cmdBlock.writeParms[0] = P38Ports[Port].svcMask;
       IssueEspCmdLeaveIntsAlone( &cmdBlock );
       cmdBlock.command = ESP_CMD_GDIPS;   
       IssueEspCmdLeaveIntsAlone( &cmdBlock );
       return;
   }

/***********************************************************************/
/* Ask ESP how much space is in ESP tx buffer. Compare this number to  */
/* number we have ready to transmit to determine actual transfer count.*/
/* GETTXSPACE has the same problem as GETRXBYTES.  We need to be sure  */
/* that the GETTXSPACE is not executed twice in a row without some     */
/* other ESI command.  The 0XFF command will guarantee that.           */
/***********************************************************************/
   cmdBlock.ioBase = P38Ports[Port].EnhancedAddress;
   cmdBlock.command = (BYTE) (0xFF); 
   IssueEspCmdLeaveIntsAlone( &cmdBlock );
   cmdBlock.command = (BYTE) (ESP_CMD_GETTXSPACE ); 
   IssueEspCmdLeaveIntsAlone( &cmdBlock );


/***********************************************************************/
/* Determine the smallest amount.                                      */
/***********************************************************************/
   P38Ports[Port].txferCount = (WORD)  
                               ((cmdBlock.uRead.val < (WORD) bytesToSend) 
                               ? cmdBlock.uRead.val : (WORD) bytesToSend);

/***********************************************************************/
/* If DMA transfer mode set up DMA controller.                         */
/***********************************************************************/
   if (P38Ports[Port].TxMode == DMA_TX)  
   {
       P38Ports[Port].DMAStatus = DMAREAD;
       SET( P38Ports[Port].flags, FX_DMA_READ );                              
       ProgramDmaController(Port, &tx_buffer[Port] [0],
                           P38Ports[Port].txferCount);
       cmdBlock.command = (BYTE) (ESP_CMD_DMATX);
       cmdBlock.ioBase = P38Ports[Port].EnhancedAddress;
       IssueEspCmdLeaveIntsAlone( &cmdBlock );

/***********************************************************************/
/* Clear ALL Tx and Rx bits in the Sid. This allows only one DMA       */
/* transfer per isr.                                                   */
/***********************************************************************/
       CLR( P38Ports[Port].svcMask, ESP_SID_TX1 );
       CLR( P38Ports[Port].Sid, ESP_SID_TXRXBOTH );
       SET( P38Ports[Port].flags, FX_ESP_BUSY );
       CLR( P38Ports[Port].flags, FX_RESET_MASK );
   }
   else    if (P38Ports[Port].TxMode == MEMORY_MAPPED)
/***********************************************************************/
/* Do Memory mapped transfer.                                          */
/***********************************************************************/
           {
               DoMemoryMapXfer(Port, &tx_buffer[Port] [0], 
                           P38Ports[Port].txferCount, FX_TRANSMIT_ON);
           }
           else
           {
/***********************************************************************/
/* It looks like programmed IO.                                        */
/***********************************************************************/
               DoProgramIOXfer(Port, &tx_buffer[Port] [0], 
                           P38Ports[Port].txferCount, FX_TRANSMIT_ON);
           }

/***********************************************************************/
/*  Update pointers and statistics type fields for Memory mapped       */  
/*  and programmed I/O now because we do not get a finish              */
/*  indicator.                                                         */  
/***********************************************************************/
       if ((P38Ports[Port].TxMode == MEMORY_MAPPED) ||
           (P38Ports[Port].TxMode == CHAR_TX))
       {
           P38Ports[Port].transmitBytes += P38Ports[Port].txferCount;
           CLR( P38Ports[Port].svcMask, ESP_SID_TX1 );
           CLR( P38Ports[Port].flags, (FX_ESP_BUSY|FX_TRANSMIT_ON));
           P38Ports[Port].NumberofTransfers++;
       }

   CLR( P38Ports[Port].flags, (FX_ESP_BUSY|FX_TRANSMIT_ON));
   return;

}


/***********************************************************************/
/*                                                                     */
/*  ReadTransferCount                                                  */
/*                                                                     */
/*     Check DMA to see how much as transfered.                        */
/*                                                                     */
/*                                                                     */
/*                                                                     */
/***********************************************************************/
WORD ReadTransferCount( WORD Port )
{
   WORD     count;
   BYTE     regTemp;



/***********************************************************************/
/* Depending on bus we do different things.                            */
/***********************************************************************/
   if (computer == AT)
   {
       outp( DMA_CLEAR, 0 );
/***********************************************************************/
/* Count register is one after the base address register.              */
/***********************************************************************/
       regTemp = (BYTE) (DMA_ADDRESS + (P38Ports[Port].dmaChannel << 1));
       regTemp++;
/***********************************************************************/
/* First read of count register provides LSB, next provides MSB.        */
/***********************************************************************/
      count = inp( regTemp );          
      count = ( (inp( regTemp ) << 8) | count );
   }
   else
   {
      outp( DMA_XFN, Get_Count + P38Ports[Port].dmaChannel );
/***********************************************************************/
/* First read provides LSB, next provides MSB.                         */
/***********************************************************************/
      count = inp( DMA_EXE );                  
      count = ( (inp( DMA_EXE ) << 8) | count );
   }  

/***********************************************************************/
/* Compensate for controller "off by 1".                               */
/***********************************************************************/
   return( count + 1 );
}


/***********************************************************************/
/*                                                                     */
/*  ProgramDmaController                                               */
/*                                                                     */
/* This code will setup the DMA controller for a subsequent transfer.  */
/*                                                                     */
/*                                                                     */
/*                                                                     */
/***********************************************************************/
void ProgramDmaController( WORD Port, char * pBuf, WORD XferCount )
{
   BYTE     regTemp;      
   LONG     physAddr;
   BYTE     channel;
   BYTE     mode ;
   WORD     count;
   WORD     Offset, Segment;
   LONG     endPhysAddr;



/***********************************************************************/
/* Store channel for fast access.                                      */
/***********************************************************************/
   channel = P38Ports[Port].dmaChannel;

/***********************************************************************/
/* Calculate physical address.                                         */
/***********************************************************************/

   Offset  = FP_OFF(pBuf);
   _asm
   {
       mov     dx, ds
       mov     Segment, dx
   }
   physAddr =  (LONG) Segment << 4;
   physAddr += (LONG) Offset;

/***********************************************************************/
/* Check for 64K-page wrap. If wrap, shorten count so that transfer    */
/* goes only to end of first page. ESP will interrupt again because    */
/* interrupting condition is still present (ESP still has data/room),  */
/* and at that time the next DMA transfer will automatically begin     */
/* with the second page.                                               */
/***********************************************************************/
   count = XferCount;
   endPhysAddr = physAddr + (LONG)count - 1;
   if ( (endPhysAddr & 0xFFFF0000) != (physAddr & 0xFFFF0000) )
      count = 0x0000 - (WORD)physAddr;

/***********************************************************************/
/* Controller chips on MicroChannel are different than standard AT.    */
/***********************************************************************/
   if (computer == AT)
   {
/***********************************************************************/
/* Set channel's mask bit to disable channel                           */
/***********************************************************************/
      outp( DMA_MASK, channel | 0x04 );
      outp( DMA_CLEAR, 0 );

/***********************************************************************/
/* Set mode to read or write.                                          */
/***********************************************************************/
      if (P38Ports[Port].flags & FX_DMA_READ)
      {
         mode = (BYTE) (channel | DMA_type_read);
      }
      else
      {
         mode = (BYTE) (channel | DMA_type_write);
      }
      outp( DMA_MODE, mode );

/***********************************************************************/
/* Set up address register. First write is LSB, next write is MSB.     */
/***********************************************************************/
      regTemp = (BYTE) (DMA_ADDRESS + (channel << 1));
      outp( regTemp, (BYTE) (physAddr & 0x000000FF ) );
      outp( regTemp, (BYTE) (( physAddr >> 8 ) & 0x000000FF ) );

/***********************************************************************/
/* Set up transfer count, which is always I/O address after address    */
/* register                                                            */
/***********************************************************************/
      regTemp++;     
      count--;
      outp( regTemp, LOBYTE(count) );
      outp( regTemp, HIBYTE(count) );

/***********************************************************************/
/* Set up page table register.                                         */
/***********************************************************************/
      if (channel == 1)
      {
         regTemp = DMA_PAGE_1;
      }
      else
      {
         regTemp = DMA_PAGE_3;
      }
      outp( regTemp, (BYTE) (( physAddr >> 16 ) & 0x000000FF ) );

/***********************************************************************/
/* Clear channel's mask bit to enable channel                          */
/***********************************************************************/
      outp( DMA_MASK, channel );
   }

   else
   {

/***********************************************************************/
/* Disable transfers on this channel.                                  */
/***********************************************************************/
      outp( DMA_XFN, Set_Chn_Mask + channel );

/***********************************************************************/
/* Set channel mode to read or write.                                  */
/***********************************************************************/
      if (P38Ports[Port].flags & FX_DMA_READ)
      {
         mode = Transfer_Data;
      }
      else
      {
         mode = Transfer_Data | Write_Mem;
      }
      outp( DMA_XFN, Set_Mode + channel );
      outp( DMA_EXE, mode );      

/***********************************************************************/
/* Set up address register                                             */
/***********************************************************************/
      outp( DMA_XFN, channel + Set_Mem_Adr );
      outp( DMA_EXE, (BYTE) ( physAddr & 0x000000FF ) );
      outp( DMA_EXE, (BYTE) (( physAddr >> 8 ) & 0x000000FF ) );
      outp( DMA_EXE, (BYTE) (( physAddr >> 16 ) & 0x000000FF ) );

/***********************************************************************/
/* Set up transfer count.                                              */
/***********************************************************************/
      count = XferCount--;
      outp( DMA_XFN, Set_Count + channel );
      outp( DMA_EXE, (BYTE) ( count & 0x00FF ) );
      outp( DMA_EXE, (BYTE) (( count >> 8 ) & 0x00FF));

/***********************************************************************/
/* Enable transfers on this channel.                                   */
/***********************************************************************/
      outp( DMA_XFN, Reset_Chn_Mask + channel );
   }

}

/***********************************************************************/
/*                                                                     */
/*                                                                     */
/* DoMemoryMapXfer                                                     */
/*     This function handles memory mapped transfers.                  */
/*                                                                     */
/*                                                                     */
/*                                                                     */
/***********************************************************************/
void DoMemoryMapXfer(WORD Port, char * UserBuffer,
                       WORD XferCount, BYTE TransferType)

{
   WORD    SegDS, SegES, OffSI, OffDI;
   BYTE    MoveType;

   if (!XferCount)         // if zero bytes to transfer then we are finished
       return;

/***********************************************************************/
/* if we are receiving data                                            */
/*     set ES:DI to equal the ESPTEST.C's buffer                       */
/*         DS:SI  to equal memory mapped address.                      */
/***********************************************************************/
   if (TransferType == FX_RECEIVE_ON)
   {
       _asm    push    ds
       _asm    pop     SegES
       OffDI = FP_OFF(UserBuffer);
       SegDS = P38Ports[Port].MemoryMapAddress;
       OffSI = 00;
       MoveType = P38Ports[Port].ReceiveMoveType;
   }

/***********************************************************************/
/* else we are transmitting data                                       */
/*     ES:DI = memory mapped buffer.                                   */
/*     DS:SI = user buffer.                                            */
/***********************************************************************/
   else
   {
       _asm    push    ds
       _asm    pop     SegDS
       OffSI = FP_OFF(UserBuffer);
       SegES = P38Ports[Port].MemoryMapAddress;
       OffDI = 00;
       MoveType = P38Ports[Port].TransmitMoveType;
   }

/***********************************************************************/
/*     Save registers and set them up for move.                        */
/***********************************************************************/
   _asm                        
   {
       push    ds
       push    es
       push    di
       push    si
       push    cx
       mov     cx, SegDS
       mov     ds, cx
       mov     cx, SegES
       mov     es, cx
       mov     di, OffDI
       mov     si, OffSI
       mov     cx, XferCount
   }
   _asm    cld                     // forward move
/***********************************************************************/
/* Is it a byte move.                                                  */
/***********************************************************************/
   if (MoveType == BYTE_IO)      
   {
       _asm    rep movsb           // do repetitive byte move
   }

/***********************************************************************/
/* it looks like a word move.                                          */
/***********************************************************************/
   else                            
   {
       _asm
       {
           test si,1               // test for odd source
           jz  DoWordMove          // even boundary
           test di,1               // test for odd destination
           jz  DoWordMove          // even boundary
           movsb                   // move one byte
           dec cx                  // decrement counter
           jz  FinishedMove

       DoWordMove:
           shr cx,1                // make our counter a word move
           rep movsw               // do repetitive word move
           jnc FinishedMove        // count was even we are finished
           movsb                   // move last byte

       FinishedMove:
       }
   }
/***********************************************************************/
/* restore used registers.                                             */
/***********************************************************************/
   _asm
   {
       pop    cx
       pop    si
       pop    di
       pop    es
       pop    ds
   }

   return;

}

/***********************************************************************/
/*                                                                     */
/*                                                                     */
/* DoProgramIOXfer                                                     */
/*                                                                     */
/*     This function handles memory mapped transfers.                  */
/*                                                                     */
/*                                                                     */
/*                                                                     */
/***********************************************************************/
void DoProgramIOXfer(WORD Port, char * UserBuffer,
                       WORD XferCount, BYTE TransferType)

{
   WORD    SegDS, SegES, OffSI, OffDI;
   WORD    IOAddress;
   BYTE    MoveType;

   if (!XferCount)         // if zero bytes to transfer then we are finished
       return;

/***********************************************************************/
/* Programmed IO address is base address plus 2.                       */
/***********************************************************************/
   IOAddress = (P38Ports[Port].EnhancedAddress + 2);  

/***********************************************************************/
/* if we are receiving data...                                         */
/*     ES:DI = user's buffer.                                          */
/***********************************************************************/
   if (TransferType == FX_RECEIVE_ON)
   {
       _asm    push    ds
       _asm    pop     SegES
       OffDI = FP_OFF(UserBuffer);
       MoveType = P38Ports[Port].ReceiveMoveType;
   }

/***********************************************************************/
/* else we are transmitting data....                                   */
/*     DS:SI = user's buffer.                                          */
/***********************************************************************/
   else
   {
       _asm    push    ds
       _asm    pop     SegDS
       OffSI = FP_OFF(UserBuffer);
       MoveType = P38Ports[Port].TransmitMoveType;
   }

/***********************************************************************/
/* save and set up registers.                                          */
/***********************************************************************/
   _asm                        
   {
       push    ds
       push    es
       push    di
       push    si
       push    cx
       push    dx
       mov     cx, SegDS
       mov     ds, cx
       mov     cx, SegES
       mov     es, cx
       mov     di, OffDI
       mov     si, OffSI
       mov     cx, XferCount
       mov     dx, IOAddress
   }

/***********************************************************************/
/* If byte io.                                                         */
/***********************************************************************/
   _asm    cld                         // forward move
   if (MoveType == BYTE_IO)       
   {
       if (TransferType == FX_RECEIVE_ON)
          _asm    rep insb             // do repetitive recv byte move
       else
          _asm    rep outsb            // do repetitive xmit byte move

   }

   else                            // it is a word io
/***********************************************************************/
/* it is word io.                                                      */
/***********************************************************************/
   {
       if (TransferType == FX_RECEIVE_ON)   
       {
           _asm
           {
               shr cx,1                // make our counter a word move
               rep insw                // do repetitive word IO
               jnc FinishedRecv        // count was even we are finished
               insb                    // do one more recv IO 
           FinishedRecv:
           }
       }
       else                                 // do word xmit IO
       {
           _asm
           {
               shr cx,1                // make our counter a word move
               rep outsw               // do repetitive xmit word move
               jnc FinishedXmit        // count was even we are finished
               outsb                    // do one xmit word IO
           FinishedXmit:
           }
        }

   }

/***********************************************************************/
/* restore registers                                                   */
/***********************************************************************/
   _asm
   {
       pop     dx   
       pop     cx
       pop     si
       pop     di
       pop     es
       pop     ds
   }

   return;

}


/***********************************************************************/
/*                                                                     */
/*                                                                     */
/* DoReceiveFilter - handle a receive buffer that has errors.          */
/*                                                                     */
/*                                                                     */
/*                                                                     */
/***********************************************************************/

WORD DoReceiveFilter(WORD Port, char *UserBuffer, 
                      WORD XferCount,  WORD NumOfErrors)
{
   WORD    Error,i,Data;
   WORD    IoAddress, ErrAddress;

   if (!XferCount)         // if zero bytes to transfer then we are finished
       return (0);

   IoAddress = P38Ports[Port].EnhancedAddress + 2;
   ErrAddress = P38Ports[Port].EnhancedAddress + 7;

   for (i = 0; (XferCount); XferCount--)
   {
   /************************************************************************/
   /*  Get the byte of data                                                */
   /*  Get the error byte status                                           */
   /*  Test for bits 0-2 for being set.                                    */
   /*  if set we have an error so increment the error flag, decrement the  */
   /*   the error count and do not store the bad byte.                     */
   /*  else store the byte and go for more.                                */
   /************************************************************************/
       Data  = inp ((WORD) IoAddress);
       Error = inp ((WORD) ErrAddress);
       if (Error & 7)
       {
           NumOfErrors--;
           if (Error & 1)
               P38Ports[Port].parityErrors++;
           if (Error & 2)
               P38Ports[Port].framingErrors++;
           if (Error & 4)
               P38Ports[Port].breakReceived = TRUE;    
       }
       else
       {
           P38Ports[Port].breakReceived = FALSE;    
           UserBuffer[i++] = (BYTE) Data;
       }
   }

   return(NumOfErrors);        // return any errors counts
                               // that were not handled
}



void IssueEspCmd( ESPCMDBLOCK *pCmdBlk )
{
   pCmdBlk->IntFlagAction = DONT_CARE;
   TheRealIssueNoWait( pCmdBlk );

}

void IssueEspCmdLeaveIntsAlone( ESPCMDBLOCK *pCmdBlk )
{
   pCmdBlk->IntFlagAction = LEAVE_INTS_ALONE;
   TheRealIssueNoWait( pCmdBlk );
}




void TheRealIssueNoWait( ESPCMDBLOCK *pCmdBlk )
{
   WORD     ioBase = pCmdBlk->ioBase;     // deref for speed
   BYTE     *pParm;
   WORD     count;

   /************************************************************************/
   /* If caller doesn't prohibit this, prevent our ISR from interrupting   */
   /* and issuing commands while we're issuing commands also.              */
   /************************************************************************/
   if (pCmdBlk->IntFlagAction != LEAVE_INTS_ALONE)
       _asm    cli

   outp( ioBase + ESP_R_CMD1, pCmdBlk->command );

   pParm = &pCmdBlk->writeParms[0];   
   for (count = EspCmdActions[ (pCmdBlk->command) & 0x7F ].numWrites; count; count-- )
   {
      outp( ioBase + ESP_R_CMD2, *pParm++ );
   }      

   /************************************************************************/
   /* Commands which return two bytes always return the MSB in STATUS1,    */
   /* and the LSB in STATUS2. We will store MSB/STATUS1 in uRead.parms[1], */
   /* and LSB/STATUS in uRead.parms[0] so that callers can access this     */
   /* data as a WORD. Unfortuately, this means that callers who wish to    */
   /* access the data as bytes will have to know that STATUS1 is in        */
   /* uRead.parms[1], and that STATUS2 in uRead.parms[0] -- which is the   */
   /* reverse of common sense.                                             */    
   /************************************************************************/
   pParm = &(pCmdBlk->uRead.parms[1]);   
   count = EspCmdActions[ (pCmdBlk->command) & 0x7F ].numReads;
   if (count >= 1)                 
   {
      *pParm-- = (BYTE) inp( ioBase + ESP_R_STATUS1 ); 
   }

   if (count >= 2)                   
   {

      *pParm = (BYTE) inp( ioBase + ESP_R_STATUS2 ); 
   }
   if (pCmdBlk->IntFlagAction != LEAVE_INTS_ALONE)
       _asm    sti
}

