/**************************************************************************
 * ESPTEST.C                                                              *
 * Tests ESP enhanced mode receive and transmit                           *
 * Requires configuration file ESPTEST.CFG                                *
 * Requires test file ESPTEST.DAT                                         *
 **************************************************************************/

/*** NOTICE *** NOTICE *** NOTICE *** NOTICE *** NOTICE *** NOTICE ***
 ***                                                               ***
DISCLAIMER: THE PRODUCT IS PROVIDED TO DEVELOPER "AS IS" WITHOUT WARRANTY
OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO, THE 
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE,
AND THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PRODUCT IS 
ASSUMED BY DEVELOPER, INCLUDING ALL COSTS OF SERVICING, REPAIRS OR CORRECTIONS.  
DEVELOPER AGREES THAT HAYES WILL NOT BE LIABLE FOR ANY DAMAGES DIRECTLY OR 
INDIRECTLY RELATED TO THE USE OF THE PRODUCT, INCLUDING BUT NOT LIMITED
TO, LOST PROFITS, INDIRECT, SPECIAL OR CONSEQUENTIAL DAMAGES, WHETHER BASED
ON CONTRACT, TORT OR ANY OTHER LEGAL THEORY.  DEVELOPER ACKNOWLEDGES THAT 
THE PRODUCT IS STILL UNDER DEVELOPMENT AND THAT HAYES DOES NOT WARRANT 
THAT THE PRODUCT IN ITS PRESENT OR FUTURE FORM WILL MEET DEVELOPER'S NEEDS.
 ***                                                               ***
 *** NOTICE *** NOTICE *** NOTICE *** NOTICE *** NOTICE *** NOTICE ***/

/***********************************************************************/
/*    Copyright 1993 Hayes Microcomputer Products, Inc.                */
/*    All rights reserved.                                             */
/*    Use of copyright notice does not imply publication.              */
/***********************************************************************/
/***********************************************************************/
/*                                                                     */
/*                                                                     */
/*     NOTE:   This code is specifically for the ESP2 board.           */
/*                                                                     */
/*                     T E S T    C O D E                              */
/*                                                                     */
/*     This code uses the Hayes Enhanced Serial Interface (ESI) Ver 2  */  
/*     to operate the ESP2 adapter.  Please refer to that manual for   */
/*     more detail on the commands.                                    */
/*                                                                     */
/*         The majority of this code is the display, compare and       */
/*         guts of the test portion of the program.                    */
/***********************************************************************/

#include <stdio.h>
#include <conio.h>
#include <memory.h>
#include <stdlib.h>
#include <string.h>
#include <dos.h>
#include <time.h>
#include "esptype.h"
#include "espfunc.h"

BYTE   RentrantFlag;
BYTE   DisplayCount;
BYTE   CompareByte = 0x14;


void main()
{
   WORD    CursorType;
   BYTE    BadKey, DebugChar;
   ESPCMDBLOCK cmdBlock;


/***********************************************************************/
/* GET CONFIGURATION FROM FILE                                         */
/***********************************************************************/
   result = getconfig();

   switch (result)
   {
       case 1:
/***********************************************************************/
/* could not find ESPTEST.CFG                                          */
/***********************************************************************/
           printf( "Need configuration file %s\n", configfile ); 
           return; 

       case 2:
/***********************************************************************/
/* problem with an entry in file.                                      */
/***********************************************************************/
           printf("File Format Error in %s\n", configfile); 
           printf(ConfigFileStr);
           printf("\n");
           return;

       case 3:
/***********************************************************************/
/* invalid value in file.                                              */
/***********************************************************************/
           printf("Invalid or no value in %s\n", configfile); 
           printf(ConfigFileStr);
           printf("\n");
           return;

       case 4:
/***********************************************************************/
/* Could not find an ESP adapter.                                      */
/***********************************************************************/
           printf("\n ESP Adapter not found.....\n"); 
           return;
      
       default:
           break;
   }

/***********************************************************************/
/* GET TEST PATTERN FROM FILE                                          */
/***********************************************************************/
   if( !getpattern() ) return;

/***********************************************************************/
/* get cursor type and save it.                                        */
/*  hide the cursor and then restore it on exit.                       */
/* Possibly put a cfg variable in here to turn this off.               */
/***********************************************************************/
   _asm
   {
       mov ah,03h          ; get cursor position
       mov bh,0            ; first page
       int 10h             ; video interrupt
       mov CursorType, cx  ; save the cursor type
       mov cx,2000h        ; hide cursor
       mov ah,01
       int 10h
       
   }

   quit = 0;

/***********************************************************************/
/*      Do one time initialization here.                               */
/***********************************************************************/
   DoEspConfig();

/***********************************************************************/
/*     let's get in a loop.                                            */
/***********************************************************************/

   while ( quit != 1 )
   {
       DisplayCount = 11;
       RentrantFlag = 0;

/***********************************************************************/
/*      Do one time initialization here.                               */
/***********************************************************************/
       InitVariables();

/***********************************************************************/
/* Flow on our local transmitter.                                      */
/***********************************************************************/
       if( P38Ports[0].PortState)
       {
           cmdBlock.ioBase = P38Ports[0].EnhancedAddress;
           cmdBlock.command = ESP_CMD_XONLOCAL;     
           IssueEspCmd( &cmdBlock );
       }
       if( P38Ports[1].PortState)
       {
           cmdBlock.ioBase = P38Ports[1].EnhancedAddress;
           cmdBlock.command = ESP_CMD_XONLOCAL;     
           IssueEspCmd( &cmdBlock );
       }

       cls();
       set_cursor_pos( 0, 0 );
       printf( "\n" );
       printf( "\n                 E S P   H A R D W A R E   T E S T");
       printf( "\n" );
       printf( "\n     0 - Transmit One Message  -  Port 1." );
       printf( "\n     1 - Transmit Continuously -  Port 1." );
       printf( "\n     2 - Transmit Continuously Do not display buffer. -  Port 1." );
       printf( "\n     3 - Receive Continuously  -  Port 1." );
       printf( "\n     4 - Receive Continuously  Do not display buffer. -  Port 1." );
       printf( "\n" );
       printf( "\n     5 - Transmit One Message  -  Port 2." );
       printf( "\n     6 - Transmit Continuously -  Port 2." );
       printf( "\n     7 - Transmit Continuously Do not display buffer. -  Port 2." );
       printf( "\n     8 - Receive Continuously  -  Port 2." );
       printf( "\n     9 - Receive Continuously  Do not display buffer. -  Port 2." );
       printf( "\n" );
       printf( "\n     A - Transmit w/Compare    -  Port 1." );
       printf( "\n     B - Receive  w/Compare    -  Port 1." );
       printf( "\n     C - Transmit w/Compare    -  Port 2." );
       printf( "\n     D - Receive  w/Compare    -  Port 2." );
       printf( "\n" );
       printf( "\n     S - Display Port Configurations.");
       printf( "\n     I - Display P38 Low level Status.");
       printf( "\n" );
       printf( "\n     Q - Quit ESP test." );

       BadKey = 1;
       while (BadKey)
       {
           key = getch();
           switch (key)
           {
               case '0':
                  TransmitPacket(1,0);
                  BadKey = 0;
                  break;

               case '1':
                  TransmitPacket(2,0);
                  BadKey = 0;
                  break;

               case '2':
                   // if we are at 57600 then time our display better
                   if (BaudRateTable[P38Ports[0].bitRate] >= 57600)
                       TransmitPacket(4,0);
                   else
                       TransmitPacket(3,0);
                  BadKey = 0;
                  break;


               case '3':
                  ReceivePacket(2,0);
                  BadKey = 0;
                  break;

               case '4':
                   // if we are at 57600 then time our display better
                   if (BaudRateTable[P38Ports[0].bitRate] >= 57600)
                       ReceivePacket(4,0);
                   else
                       ReceivePacket(3,0);
                  BadKey = 0;
                  break;

               case '5':
                   TransmitPacket(1,1);
                   BadKey = 0;
                   break;

               case '6':
                   TransmitPacket(2,1);
                   BadKey = 0;
                   break;

               case '7':
                   // if we are at 57600 then time our display better
                   if (BaudRateTable[P38Ports[0].bitRate] >= (WORD) 57600)
                       TransmitPacket(4,1);
                   else
                       TransmitPacket(3,1);
                   BadKey = 0;
                   break;


               case '8':
                   ReceivePacket(2,1);
                   BadKey = 0;
                   break;

               case '9':
                          // if we are at 57600 then time our display better
                   if (BaudRateTable[P38Ports[0].bitRate] >= 57600)
                       ReceivePacket(4,1);
                   else
                       ReceivePacket(3,1);
                   BadKey = 0;
                   break;

               case 'A':
               case 'a':
                  TransmitandCompare(0);
                  BadKey = 0;
                  break;

               case 'B':
               case 'b':
                   CompareByte = CompareStartCharacter[0];
                   ReceiveandCompare(0);
                   BadKey = 0;
                   break;

               case 'C':
               case 'c':
                  TransmitandCompare(1);
                  BadKey = 0;
                  break;

               case 'D':
               case 'd':
                  CompareByte = CompareStartCharacter[1];
                  ReceiveandCompare(1);
                  BadKey = 0;
                  break;

               case 'S':
               case 's':
                   DisplayConfiguration();    
                   BadKey = 0;
                   break;
 
               case 'I':
               case 'i':
                   DisplayLowLevelStatus();    
                   BadKey = 0;
                   break;
 
               case 'Q':
               case 'q':
                  BadKey = 0;
                  quit = 1;
                  break;

               case 'U':
               case 'u':
                   BadKey = 0;
                   _asm    int 3
                   DebugChar = P38Ports[0].rcvbfrStart[0];
                   break;
           }

       }
   }

/***********************************************************************/
/* Reset IRQ's and vectors.                                            */
/***********************************************************************/
   ResetIRQVector();
   if( P38Ports[0].PortState)
   {
       cmdBlock.ioBase = P38Ports[0].EnhancedAddress;
       cmdBlock.command = ESP_CMD_RESET; 
       IssueEspCmd( &cmdBlock );
   }
   if( P38Ports[1].PortState)
   {
       cmdBlock.ioBase = P38Ports[1].EnhancedAddress;
       cmdBlock.command = ESP_CMD_RESET; 
       IssueEspCmd( &cmdBlock );
   }

/***********************************************************************/
/*  reset the cursor                                                   */
/***********************************************************************/
   _asm
   {
       mov ah,01h          ; get cursor position
       mov cx, CursorType  ; get the cursor type
       int 10h             ; video interrupt
   }
   cls();
   set_cursor_pos( 0, 0 );

}


/* ======================= SET_CURSOR_POS ===================================
 * Sets screen cursor position                                        
 */
void set_cursor_pos( row, col )
   unsigned char        row;
   unsigned char        col;
{
   union REGS           regs;

   regs.h.ah = 0x02;              /* function: set cursor position */
   regs.h.bh = 0;                 /* video page 0 */
   regs.h.dh = row;
   regs.h.dl = col;
   int86( 0x10, &regs, &regs );   /* INT 10H is BIOS Video */
}

/* ============================= CLS ========================================
 * Clears entire screen                                               
 */
void cls(void)
{
   union REGS          regs;

   regs.h.ah = 0x06;              /* functon: scroll */
   regs.h.al = 0x00;              /* scroll entire screen */
   regs.h.bh = 0x07;              /* normal attribute (black & white) */
   regs.x.cx = 0x00;              /* FROM 00,00 */
   regs.x.dx = 0x184F;            /* TO   24,79 */   
   int86( 0x10, &regs, &regs );   /* INT 10H is BIOS Video */
}


/* ============================= CLS ========================================
 * Clears entire screen                                               
 */
void clsPart(BYTE StartRow, BYTE StartCol,
           BYTE EndRow, BYTE EndCol, BYTE LinesRolled)
{
   union REGS          regs;

   regs.h.ah = 0x06;              /* functon: scroll */
   regs.h.al = LinesRolled;       /* scroll entire screen */
   regs.h.bh = 0x07;              /* normal attribute (black & white) */
   regs.x.cx = 0x0100;            /* FROM 01,00 */
   regs.h.ch = StartRow;          /* row past in by caller  */
   regs.h.cl = StartCol;          /* TO  79 */   
   regs.h.dh = EndRow;            /* row past in by caller  */
   regs.h.dl = EndCol;            /* TO  79 */   
   int86( 0x10, &regs, &regs );   /* INT 10H is BIOS Video */
}



/* ================================ SERVICE_KEY =============================
 * Services keyboard input
 * ENTER:       None
 * RETURN:      None
 */
void service_key(void)
{

   if( key == 'Q' || key == 'q' )      /* quit esp */
      quit = 1;

}






/* ================================ TransmitPacket =============================
 * Transmits and displays packet.
 * ENTER:       Port = 1 or 2
 * RETURN:      None
 */
void   TransmitPacket(BYTE TransmitType, BYTE Port)

{
   ESPCMDBLOCK cmdBlock;
   BYTE    ServiceMask;
   time_t  StartTime, EndTime;
   ULONG    TotalData;

//   _asm    int 3

   quit = 0;
   cls();
   set_cursor_pos( 0, 0 );

   printf("\t\tTransmit Packet and Status Display    Press [Q] to quit..");


/***********************************************************************/
/* turn on the service mask for RX and error SID's.                    */
/***********************************************************************/
   if( P38Ports[Port].PortState)
   {
       SetServiceMask (Port, 0);
       SET (P38Ports[Port].svcMask, ESP_SID_PORT1);  
       CLR (P38Ports[Port].svcMask, ESP_SID_TX1);   // clear transmit bit.
       CLR (P38Ports[Port].flags, FX_TRANSMIT_ON);
       SetServiceMask (Port, P38Ports[Port].svcMask);
   }
   else
   {
       printf("\n Port %d not available.... ", Port);
       return;
   }

/***********************************************************************/
/* Setup the starting time so that we can calculate thruput.           */
/***********************************************************************/
   TotalData = 0;
   time (&StartTime);
   do 
   {
   /***********************************************************************/
/* check keyboard for a 'Q' or 'P' key.                                */
/***********************************************************************/
       if( kbhit() )
       {
           key = getch();
           if( key == 'P' || key == 'p' )      
           {
               set_cursor_pos(19,10);
               printf("Transmitting has been paused... Press any key to continue.");
               getch();
           }
           else
               service_key();
       }

/***********************************************************************/
/* Test to see if we are already transmitting. If yes, then restart the*/
/* loop.                                                               */
/***********************************************************************/
       if (P38Ports[Port].flags & FX_TRANSMIT_ON)  
       {
/***********************************************************************/
/* If we are polling then we call directly into the ESP ISR routine.   */
/***********************************************************************/
           if (!P38Ports[Port].IRQValues->ESPIrq)   
           {
               ESPIsr(Port);
               continue;
           }
       }
       ServiceMask = (BYTE) P38Ports[Port].svcMask;

/***********************************************************************/
/* Turn off interrupts.  Set the TX in the service mask (if not there) */
/* if our current service mask is not equal to the store service mask  */
/* then issue a set service mask.                                      */
/***********************************************************************/
       _asm    cli                                      
       SET (P38Ports[Port].flags, FX_TRANSMIT_ON);
       if (!(P38Ports[Port].svcMask & ESP_SID_TX1))
           ServiceMask |= ESP_SID_TX1;
       if (P38Ports[Port].svcMask != ServiceMask)
       {
           cmdBlock.writeParms[0] = P38Ports[Port].svcMask = ServiceMask;       if (!(P38Ports[Port].svcMask & ESP_SID_TX1))
           ServiceMask |= ESP_SID_TX1;
           SetServiceMask (Port, P38Ports[Port].svcMask);
       }
       _asm    sti

/***********************************************************************/
/* Display transmit statistics and data.                               */
/***********************************************************************/
       DisplayTransmitPacket((BYTE) Port, &tx_buffer[Port] [0], 
                             TransmitType, &TotalData,0);
       
/***********************************************************************/
/* Get time and determine if 60 seconds has elapsed.  If it has calcu- */
/* late the throughput.                                                */
/***********************************************************************/
       time (&EndTime);
       if (EndTime >= (StartTime + 60))
       {
           if (TotalData)
           {
               set_cursor_pos(23,0);
               printf("Avg. Bytes/Sec. = %li", (TotalData/60));
               TotalData = 0;
               time (&StartTime);
           }
       }
/***********************************************************************/
/* if one transmit then we are finished.                               */
/***********************************************************************/
       if (TransmitType == 0)          
       {
           quit = 1;
       }

/***********************************************************************/
/* check keyboard for a 'Q' or 'P' key.                                */
/***********************************************************************/
       if( kbhit() )
       {
           key = getch();
           if( key == 'P' || key == 'p' )      
           {
               set_cursor_pos(19,10);
               printf("Transmitting has been paused... Press any key to continue.");
               getch();
           }
           else
               service_key();
       }

   } while ( quit != 1 );

   quit = 0;
   return;
}


/* ================================ DisplayTransmitPacket ==================
 * Displays packet data 
 * ENTER:       Port = 0 or 1, Buffer = buffer to display,
 *              Bufsize = buffer size, Pause = whether to pause between screens
 * RETURN:      None
 */

void   DisplayTransmitPacket(BYTE  Port, char * Buffer, BYTE Pause, 
                               ULONG *TotalData, BYTE JustStatus)

{
   BYTE    Row = 18;
   BYTE    Column = 0;
   int     i,j,BufCount, Size;
   ULONG   Count =  0;
   ULONG   NumberXfers= 0;
   BYTE    Error1, Error2;
   static  BYTE    FirstTime = 1;
   WORD    Sid;
   ULONG   Isr = 0;


   BufCount = 0;
   set_cursor_pos(21, 0 );

/***********************************************************************/
/* get values that are set during the ISR.  We disable interrupts so   */
/* that our info is valid.                                             */
/***********************************************************************/
   _asm    cli
   Error1 = P38Ports[Port].Error1;
   Error2 = P38Ports[Port].Error2;
   Sid = P38Ports[Port].SidAccumlative;
   Size = P38Ports[Port].txferCount;
   *TotalData += Size;
   P38Ports[Port].SidAccumlative = 0;
   P38Ports[Port].Error1 = P38Ports[Port].Error2 = 0;
   Count   = P38Ports[Port].transmitBytes;
   NumberXfers = P38Ports[Port].NumberofTransfers;
   Isr =  P38Ports[Port].ISREntry;
   _asm    sti


/***********************************************************************/
/* if pause < 4  or pause == 4 and display Count is > 10 we will       */
/* display statistics.                                                 */
/***********************************************************************/
   if ( (Pause < 4)    ||
       (Pause == 4) && (DisplayCount >= 10))
   {
       printf( " port %d   " , (Port + 1));
       printf( " error status 1+2 = %.2X  %.2x hex   Total Transfers  = %li\n",
            Error1, Error2, NumberXfers);
       printf( " service ID = %.2Xh", Sid);
       printf( "         MSR = %.4lu              RTO = %.4lu", 
                   P38Ports[Port].ModemStatusRegister, 
                   P38Ports[Port].ReceiveToutCtr);
       set_cursor_pos(23, 26 );
       printf( "ISR = %.4lu\n", Isr);
       set_cursor_pos(23, 48 );
       printf( "Bytes = %lu \n", Count);
       DisplayCount = 0;
   }
   else
       DisplayCount++;


/***********************************************************************/
/* if pause is > than 2 then we are in fast receive mode and do not    */
/* display buffer data.                                                */
/***********************************************************************/
   if ((Pause > 2) || (JustStatus))
       return;

   if (FirstTime)
   {
       FirstTime = 0;
       clsPart(1,0,20,79,0);
   }

/***********************************************************************/
/* This loop will display all the current transmit data.               */
/***********************************************************************/
   while   (BufCount < Size)
   {
       set_cursor_pos(Row,Column);
       for (j=i=0; (i<81 && BufCount < Size)  ; BufCount++, i++)
       {
           if (Buffer[BufCount] == 0x0d)
           {
               BufCount++;
               BufCount++;
               clsPart(1,0,20,79,1);
               set_cursor_pos(Row,Column);
               i = 0;
               break;
           }
           if (i >= 80)
           {
               clsPart(1,0,20,79,1);
               set_cursor_pos(Row,Column);
               i = 0;
               break;
           }
           printf("%c",  Buffer[BufCount]);

       }    
   }
   if (Pause == 1)
   {
       set_cursor_pos(18,10);
       printf("\n\n Press any key to continue...");
       key = getch();
       service_key();
       if (quit == 1)
           return;
   }
   
   return;
}




/* ================================ ReceivePacket =============================
 * Receives and displays packet.
 * ENTER:       ReceiveType = 1 or 2 or 3
 * RETURN:      None
 */
void   ReceivePacket(BYTE ReceiveType, BYTE Port)

{
   ESPCMDBLOCK cmdBlock;
   BYTE    ServiceMask;
   BYTE    StartDisplay = 0;
   time_t  StartTime, EndTime;
   ULONG    TotalData;

   quit = 0;
   cls();

   set_cursor_pos( 0, 0 );
     /* SETUP */

   printf("\t\tReceive Packet and Status Display    Press [Q] to quit..");


/***********************************************************************/
/* Set up for receiving.                                               */
/***********************************************************************/

   if( P38Ports[Port].PortState)
   {
       memset( rx_buffer[0], '#', P38Ports[Port].rcvbfrSize );
       P38Ports[Port].rxHolding = FALSE;
       SetServiceMask (Port, 0);
       CLR (P38Ports[Port].flags, FX_RECEIVE_ON);
       SET (P38Ports[Port].svcMask, ESP_SID_PORT1);  
       CLR (P38Ports[Port].svcMask, ESP_SID_TX1);   // clear transmit bit.
       cmdBlock.ioBase = P38Ports[Port].EnhancedAddress;
       cmdBlock.writeParms[0] = P38Ports[Port].svcMask;
       SetServiceMask (Port, P38Ports[Port].svcMask);
   }
   else
   {
       printf("\n Port %d not available....",Port);
       return;
   }


/***********************************************************************/
/* Set up starting time so that we can calculate throughput.           */
/***********************************************************************/
   TotalData = 0;
   time (&StartTime);
   do 
   {
       if( kbhit() )
       {
           key = getch();
           service_key();
           if (quit == 1)
               break;
       }
/***********************************************************************/
/* if this is not set we do not have data                              */
/***********************************************************************/
       if (!(P38Ports[Port].flags & FX_RECEIVE_ON))
       {
           if (!P38Ports[Port].IRQValues->ESPIrq)      // POLLING 
           {
               ESPIsr(Port);
               continue;
           }
       }
/***********************************************************************/
/* Display receive data.                                               */
/***********************************************************************/
       DisplayReceivePacket((BYTE) (Port), ReceiveType, 
                   StartDisplay, &TotalData, 0);
       StartDisplay = 1;

/***********************************************************************/
/* calculate time to determine if 60 seconds have elapsed.  If so we   */
/* calculate throughput.                                               */
/***********************************************************************/
       time (&EndTime);
       if (EndTime >= (StartTime + 60))
       {
           if (TotalData)
           {
               set_cursor_pos(23,0);
               printf("Avg. Bytes/Sec. = %li", (TotalData/60));
               TotalData = 0;
               time (&StartTime);
           }
       }

/***********************************************************************/
/* figure out service mask and do a SET ServiceID if the current and   */
/* SID are different.                                                  */
/***********************************************************************/
       _asm    cli
       ServiceMask = (BYTE) P38Ports[Port].svcMask;
       CLR (P38Ports[Port].flags, FX_RECEIVE_ON);
       if (!(P38Ports[Port].svcMask & ESP_SID_RX1))
               ServiceMask |= ESP_SID_RX1;
       if (P38Ports[Port].svcMask != ServiceMask)
       {
           cmdBlock.writeParms[0] = P38Ports[Port].svcMask = ServiceMask;
           SetServiceMask (Port, P38Ports[Port].svcMask);
       }
       _asm    sti

   } while ( quit != 1 );

   quit = 0;
   return;
}



/* ================================ DisplayReceivePacket ===================
 * Displays packet data 
 * ENTER:       Port = 0 or 1, Buffer = buffer to display,
 *              Bufsize = buffer size, Pause = whether to pause between screens
 * RETURN:      None
 */

void   DisplayReceivePacket(BYTE Port, BYTE Pause, BYTE StartDisplay,
                               ULONG *TotalData, BYTE JustStatus)

{
   int     XferCount;
   ULONG    Count =  0;
   ULONG   NumberXfers = 0;
   ULONG   Isr = 0;
   signed  int size, length, lengthx;
   BYTE    Error1, Error2;
   WORD    Sid;
   char    *pInt, *pBfr;

   set_cursor_pos(21, 0 );

/***********************************************************************/
/* get values that are set during the ISR. Turn off interrupts to get  */
/* the data.                                                           */
/***********************************************************************/
   _asm    cli
   Error1 = P38Ports[Port].Error1;
   Error2 = P38Ports[Port].Error2;
   Sid = P38Ports[Port].SidAccumlative;
   XferCount = P38Ports[Port].rxferCount;
   P38Ports[Port].SidAccumlative = 0;
   P38Ports[Port].Error1 = P38Ports[Port].Error2 = 0;
   Count   = P38Ports[Port].receiveBytes;
   pInt = P38Ports[Port].rcvbfrDrvr;
   pBfr = P38Ports[Port].rcvbfrPC;
   NumberXfers = P38Ports[Port].NumberofTransfers;
   Isr =  P38Ports[Port].ISREntry;
   _asm    sti

/***********************************************************************/
/* figure out what the length of receive data.                         */
/***********************************************************************/
   length = P38Ports[Port].rcvbfrSize; 
   lengthx = 0;
   size = pInt - pBfr;
   if (size < 0)
       size += P38Ports[Port].rcvbfrSize;
   if (length > size)
       length = size;
   *TotalData += size;

/***********************************************************************/
/* Display statistics every time except for when our Pause == 4 and our*/
/* DisplayCount is < 10.                                               */
/***********************************************************************/
   if ( (Pause < 4)    ||
       (Pause == 4) && (DisplayCount >= 10))
   {
       printf( " port %d   " , (Port + 1));
       printf( " error status 1+2 = %.2X  %.2x hex   Total Transfers  = %li\n",
                Error1, Error2, NumberXfers);
       printf( " service ID = %.2Xh", Sid);
       printf( "         MSR = %.4lu              RTO = %.4lu", 
                   P38Ports[Port].ModemStatusRegister, 
                   P38Ports[Port].ReceiveToutCtr);
       set_cursor_pos(23, 26 );
       printf( "ISR = %.4lu\n", Isr);
       set_cursor_pos(23, 48 );
       printf( "Bytes = %lu \n", Count);
       DisplayCount = 0;

   }
   else
       DisplayCount++;

   if (JustStatus)
       return;
/***********************************************************************/
/* This loop is responsible for calculating the new buffer pointer so  */
/* that the receiver can use the buffer and it will display receive    */
/* data if necessary.                                                  */
/***********************************************************************/
   while (length)
   {
       if (pInt > pBfr)
       {
           size = pInt - pBfr;
           if (size > length)
               size = length;
           if (Pause < 3)
               DisplayIt(pBfr, size, Pause, StartDisplay);
           pBfr += size;
       }
       else    if (pInt < pBfr)
       {
           size = P38Ports[Port].rcvbfrEnd - pBfr + 1;
           if (size > length)
               size = length;
           if (Pause < 3)
               DisplayIt(pBfr, size, Pause, StartDisplay);
           pBfr += size;
           if (pBfr > P38Ports[Port].rcvbfrEnd)
               pBfr = P38Ports[Port].rcvbfrStart;
       }
       else
           break;
       length -= size;

   }
    
   P38Ports[Port].rxHolding = FALSE;
   P38Ports[Port].rcvbfrPC = pBfr;

#ifdef j
   if (P38Ports[Port].rxHolding)
   {
/***********************************************************************/
/* ISR is waiting on us to remove data from queue. Since we've just    */
/* removed data, see if there is now room in the receive queue for     */
/* trigger_level bytes. If so, reenable RxFIFO interrupt for this      */
/* port so ISR can continue to stuff incoming data.                    */
/***********************************************************************/
      size = P38Ports[Port].rcvbfrDrvr - P38Ports[Port].rcvbfrPC;
      if (size < 0)
         size += P38Ports[Port].rcvbfrSize;
      length = P38Ports[Port].rcvbfrSize - size - 1;
      if ((WORD) length >= P38Ports[Port].rxTrigger)
      {
         P38Ports[Port].rxHolding = FALSE;
      }
   }
#endif

   return;
}  


/***********************************************************************/
/* This routine display the actual receive data.                       */
/***********************************************************************/
void   DisplayIt(char *pBfr, WORD Size, BYTE Pause, BYTE StartDisplay)

{
   static  BYTE    Row = 18;
   static  BYTE    Column = 0;
   int     i,j;
   char    *pEnd;


   if (!StartDisplay)
   {
       Row = 18;
       Column = 0;
   }

   pEnd = (pBfr + Size);

   while   (pBfr < pEnd)
   {
       set_cursor_pos(Row,Column);
       for (j=i=0; (Column < 81 && pBfr < pEnd); pBfr++, i++, Column++)
       {

           if (*pBfr == 0x0a)
           {
               Column--;            // decrement loop counter by one.
               continue;
           }

           if (*pBfr == 0x0d)
           {
               pBfr++;
               if (*pBfr == 0x0a)
                  pBfr++;
               Column = 0;
               if (Pause == 1)
               {
                   printf("\n\n Press any key to continue...");
                   key = getch();
                   service_key();
                   if (quit == 1)
                       return;

               }    
               clsPart(1,0,20,79,1);
               set_cursor_pos(Row,Column);
               break;
           }
           if (Column >= 80)
           {
               Column = 0;
               if (Pause == 1)
               {
                   printf("\n\n Press any key to continue...");
                   key = getch();
                   service_key();
                   if (quit == 1)
                      return;
 
               }    
               clsPart(1,0,20,79,1);
               set_cursor_pos(Row,Column);
               break;
           }

           printf("%c",  *pBfr);
               
       }    


   }

   if (Pause == 1)
   {
       set_cursor_pos(19,0);
       printf("\n\n Press any key to continue...");
       key = getch();
       service_key();
       if (quit == 1)
           return;
   }
   
   return;
}


/* ============================ TransmitandCompare =========================
 * Transmits and compares packet.
 * ENTER:       Port = 1 
 * RETURN:      None
 */
void TransmitandCompare(BYTE Port)

{
   ESPCMDBLOCK cmdBlock;
   BYTE    ServiceMask;
   time_t  StartTime, EndTime;
   ULONG    TotalData;
   WORD    j,i;


   quit = 0;
   cls();
   set_cursor_pos( 0, 0 );

   printf("\t\tTransmit Packet and Compare Display    Press [Q] to quit..");

//   _asm    int 3
   for (i = CompareStartCharacter[Port], j = 0; i < 0x100; i++, j++)
       tx_buffer[Port][j] = (BYTE) i;
   P38Ports[Port].xmtbfrSize = (j);
/***********************************************************************/
/* turn on the service mask for RX and error SID's.                    */
/***********************************************************************/
   SetServiceMask (Port,  0);
   SET (P38Ports[Port].svcMask, ESP_SID_PORT1);  
   CLR (P38Ports[Port].svcMask, ESP_SID_TX1);   // clear transmit bit.
   CLR (P38Ports[Port].flags, FX_TRANSMIT_ON);
   SetServiceMask (Port, P38Ports[Port].svcMask);

/***********************************************************************/
/* Setup the starting time so that we can calculate thruput.           */
/***********************************************************************/
   TotalData = 0;
   time (&StartTime);
   do 
   {
   /***********************************************************************/
/* check keyboard for a 'Q' or 'P' key.                                */
/***********************************************************************/
       if( kbhit() )
       {
           key = getch();
           if( key == 'P' || key == 'p' )      
           {
               set_cursor_pos(19,10);
               printf("Transmitting has been paused... Press any key to continue.");
               getch();
           }
           else
               service_key();
       }

/***********************************************************************/
/* Test to see if we are already transmitting. If yes, then restart the*/
/* loop.                                                               */
/***********************************************************************/
       if (P38Ports[Port].flags & FX_TRANSMIT_ON)  
       {
           if (!P38Ports[Port].IRQValues->ESPIrq)      // POLLING 
           {
               ESPIsr(Port);
               continue;
           }
       }
       ServiceMask = (BYTE) P38Ports[Port].svcMask;

/***********************************************************************/
/* Turn off interrupts.  Set the TX in the service mask (if not there) */
/* if our current service mask is not equal to the store service mask  */
/* then issue a set service mask.                                      */
/***********************************************************************/
       _asm    cli                                      
       SET (P38Ports[Port].flags, FX_TRANSMIT_ON);
       if (!(P38Ports[Port].svcMask & ESP_SID_TX1))
           ServiceMask |= ESP_SID_TX1;
       if (P38Ports[Port].svcMask != ServiceMask)
       {
           cmdBlock.writeParms[0] = P38Ports[Port].svcMask = ServiceMask;
           SetServiceMask (Port, P38Ports[Port].svcMask);
       }
       _asm    sti

/***********************************************************************/
/* Get time and determine if 60 seconds has elapsed.  If it has calcu- */
/* late the throughput.                                                */
/***********************************************************************/
       time (&EndTime);
       if (EndTime >= (StartTime + 60))
       {
           if (TotalData)
           {
               set_cursor_pos(23,0);
               printf("Avg. Bytes/Sec. = %li", (TotalData/60));
               TotalData = 0;
               time (&StartTime);
           }
       }

/***********************************************************************/
/* check keyboard for a 'Q' or 'P' key.                                */
/***********************************************************************/
       if( kbhit() )
       {
           key = getch();
           if( key == 'P' || key == 'p' )      
           {
               set_cursor_pos(19,10);
               printf("Transmitting has been paused... Press any key to continue.");
               getch();
           }
           else
               service_key();
       }
   } while ( quit != 1 );

   quit = 0;
   return;
}


/* ========================= ReceiveandCompare ============================
 * Receives and compares data.
 * ENTER:
 * RETURN:      None
 */
void   ReceiveandCompare(BYTE Port)
{
   ESPCMDBLOCK cmdBlock;
   BYTE    ServiceMask;
   BYTE    StartDisplay = 0;
   time_t  StartTime, EndTime;
   ULONG    TotalData;

   quit = 0;
   cls();

   set_cursor_pos( 0, 0 );
     /* SETUP */

   printf("\t\tReceive Packet and Compare Display    Press [Q] to quit..");


/***********************************************************************/
/* Set up for receiving.                                               */
/***********************************************************************/

   memset( rx_buffer[0], '#', P38Ports[Port].rcvbfrSize );
   P38Ports[Port].rxHolding = FALSE;
   SetServiceMask (Port, 0);
   CLR (P38Ports[Port].flags, FX_RECEIVE_ON);
   SET (P38Ports[Port].svcMask, ESP_SID_PORT1);
   CLR (P38Ports[Port].svcMask, ESP_SID_TX1);   // clear transmit bit.
   SetServiceMask (Port, P38Ports[Port].svcMask);

/***********************************************************************/
/* Set up starting time so that we can calculate throughput.           */
/***********************************************************************/
   TotalData = 0;
   time (&StartTime);
   do 
   {
       if( kbhit() )
       {
           key = getch();
           service_key();
           if (quit == 1)
               break;
       }
/***********************************************************************/
/* if this is not set we do not have data                              */
/***********************************************************************/
       if (!(P38Ports[Port].flags & FX_RECEIVE_ON))
       {
           if (!P38Ports[Port].IRQValues->ESPIrq)      // POLLING 
           {
               ESPIsr(Port);
               continue;
           }
       }
/***********************************************************************/
/* Compare receive data.                                                 */
/***********************************************************************/
       CompareReceivePacket((BYTE) Port, &TotalData);

/***********************************************************************/
/* calculate time to determine if 60 seconds have elapsed.  If so we   */
/* calculate throughput.                                               */
/***********************************************************************/
       time (&EndTime);
       if (EndTime >= (StartTime + 60))
       {
           if (TotalData)
           {
               set_cursor_pos(23,0);
               printf("Avg. Bytes/Sec. = %li", (TotalData/60));
               TotalData = 0;
               time (&StartTime);
           }
       }

/***********************************************************************/
/* figure out service mask and do a SET ServiceID if the current and   */
/* SID are different.                                                  */
/***********************************************************************/
       _asm    cli
       ServiceMask = (BYTE) P38Ports[Port].svcMask;
       CLR (P38Ports[Port].flags, FX_RECEIVE_ON);
       if (!(P38Ports[Port].svcMask & ESP_SID_RX1))
               ServiceMask |= ESP_SID_RX1;
       if (P38Ports[Port].svcMask != ServiceMask)
       {
           cmdBlock.writeParms[0] = P38Ports[Port].svcMask = ServiceMask;
           SetServiceMask (Port, P38Ports[Port].svcMask);
       }
       _asm    sti


   } while ( quit != 1 );

   quit = 0;
   return;
}



/* ============================ CompareReceivePacket ===================
 * Compares packet data 
 * ENTER:    
 * RETURN:      None
 */

void  CompareReceivePacket(BYTE Port, ULONG *TotalData)

{
   int     XferCount;
   ULONG    Count =  0;
   ULONG   Isr = 0;
   ULONG   NumberXfers = 0;
   signed  int size, length, lengthx;
   BYTE    Error1, Error2;
   WORD    Sid;
   char    *pInt, *pBfr;

   set_cursor_pos(21, 0 );

/***********************************************************************/
/* get values that are set during the ISR. Turn off interrupts to get  */
/* the data.                                                           */
/***********************************************************************/
   _asm    cli
   Error1 = P38Ports[Port].Error1;
   Error2 = P38Ports[Port].Error2;
   Sid = P38Ports[Port].SidAccumlative;
   XferCount = P38Ports[Port].rxferCount;
   P38Ports[Port].SidAccumlative = 0;
   P38Ports[Port].Error1 = P38Ports[Port].Error2 = 0;
   Count   = P38Ports[Port].receiveBytes;
   pInt = P38Ports[Port].rcvbfrDrvr;
   pBfr = P38Ports[Port].rcvbfrPC;
   NumberXfers = P38Ports[Port].NumberofTransfers ;
   Isr =  P38Ports[Port].ISREntry;
   _asm    sti

/***********************************************************************/
/* figure out what the length of receive data.                         */
/***********************************************************************/
   length = P38Ports[Port].rcvbfrSize; 
   lengthx = 0;
   size = pInt - pBfr;
   if (size < 0)
       size += P38Ports[Port].rcvbfrSize;
   if (length > size)
       length = size;
   *TotalData += size;

/***********************************************************************/
/* Display statistics every time except for when our Pause == 4 and our*/
/* DisplayCount is < 10.                                               */
/***********************************************************************/
   printf( " error status 1+2 = %.2X  %.2x hex   Total Transfers  = %li\n",
        Error1, Error2, NumberXfers);
   printf( " service ID = %.2Xh", Sid);
   printf( "         MSR = %.4lu              RTO = %.4lu", 
              P38Ports[Port].ModemStatusRegister, 
              P38Ports[Port].ReceiveToutCtr);
   set_cursor_pos(23, 26 );
   printf( "ISR = %.4lu\n", Isr);
   set_cursor_pos(23, 48 );
   printf( "Bytes = %lu \n", Count);
   DisplayCount = 0;



/***********************************************************************/
/* This loop is responsible for calculating the new buffer pointer so  */
/* that the receiver can use the buffer and it will display receive    */
/* data if necessary.                                                  */
/***********************************************************************/
   while (length)
   {
       if (pInt > pBfr)
       {
           size = pInt - pBfr;
           if (size > length)
               size = length;
           CompareIt(pBfr, size, Port);
           pBfr += size;
       }
       else    if (pInt < pBfr)
       {
           size = P38Ports[Port].rcvbfrEnd - pBfr + 1;
           if (size > length)
               size = length;
           CompareIt(pBfr, size, Port);
           pBfr += size;
           if (pBfr > P38Ports[Port].rcvbfrEnd)
               pBfr = P38Ports[Port].rcvbfrStart;
       }
       else
           break;
       length -= size;

   }
    
   P38Ports[Port].rxHolding = FALSE;
   P38Ports[Port].rcvbfrPC = pBfr;

#ifdef j
   if (P38Ports[Port].rxHolding)
   {
/***********************************************************************/
/* ISR is waiting on us to remove data from queue. Since we've just    */
/* removed data, see if there is now room in the receive queue for     */
/* trigger_level bytes. If so, reenable RxFIFO interrupt for this      */
/* port so ISR can continue to stuff incoming data.                    */
/***********************************************************************/
      size = P38Ports[Port].rcvbfrDrvr - P38Ports[Port].rcvbfrPC;
      if (size < 0)
         size += P38Ports[Port].rcvbfrSize;
      length = P38Ports[Port].rcvbfrSize - size - 1;
      if ((WORD) length >= P38Ports[Port].rxTrigger)
      {
         P38Ports[Port].rxHolding = FALSE;
      }
   }
#endif

   return;
}





/***********************************************************************/
/* This routine compares the actual receive data.                      */
/***********************************************************************/
void   CompareIt(char *pBfr, WORD Size, BYTE Port)
{
   static  BYTE    FirstTime = 1;
   static  ULONG    CompareErrors = 0;
   char    *pEnd;
   char    BadData;

   if (FirstTime)
   {
       FirstTime = 0;
       set_cursor_pos(15,10);
       printf("# of compare errors: %lu", CompareErrors);
   }
   pEnd = (pBfr + Size);

   while   (pBfr < pEnd)
   {
       if ((BYTE) *pBfr != (BYTE) CompareByte)
       {
           _asm     int 3
           BadData = *pBfr;
           CompareErrors++;
           set_cursor_pos(15,10);
           printf("# of compare errors: %li", CompareErrors);
       }    
       pBfr++;
       if (CompareByte == 0xff)
           CompareByte = CompareStartCharacter[Port];
       else
           CompareByte++;

   }
   
   return;
}


/* ============================ DisplayLowLevelStatus ===================
 * Shows ISR activity at standstill. 
 * ENTER:    
 * RETURN:      None
 */


void   DisplayLowLevelStatus(void)
{

   ULONG   Isr = 0;
   BYTE    Error1, Error2;
   WORD    Sid;

   quit = 0;
   cls();

   set_cursor_pos( 0, 0 );
     /* SETUP */

   printf("\t\tLow Level Status   Press [Q] to quit..");

   set_cursor_pos(5, 20 );

   printf("Port 1 - %.4x", 
       P38Ports[0].EnhancedAddress);

   set_cursor_pos(7, 0 );
   printf( "   Error status 1+2      =\n");
   printf( "   ISR Entrancy          =\n");
   printf( "   Service ID            =\n");
   printf( "   Modem status change   =\n");
   printf( "   Receive Time outs     =\n");



   if( P38Ports[1].PortState)
   {
       set_cursor_pos(5, 50 );

       printf("Port 2 - %.4x", 
           P38Ports[1].EnhancedAddress);
   }


/***********************************************************************/
/* get values that are set during the ISR. Turn off interrupts to get  */
/* the data.                                                           */
/***********************************************************************/
   do 
   {
       if( kbhit() )
       {
           key = getch();
           service_key();
           if (quit == 1)
               break;
       }

       _asm    cli
       P38Ports[0].rcvbfrDrvr     = P38Ports[0].rcvbfrStart;
       P38Ports[0].rcvbfrPC       = P38Ports[0].rcvbfrStart;
       Error1 = P38Ports[0].Error1;
       Error2 = P38Ports[0].Error2;
       Sid = P38Ports[0].SidAccumlative;
       P38Ports[0].SidAccumlative = 0;
       P38Ports[0].Error1 = P38Ports[0].Error2 = 0;
       Isr =  P38Ports[0].ISREntry;
       _asm    sti

       set_cursor_pos(7, 28 );
       printf( "%.2X  %.2x", Error1, Error2);
       set_cursor_pos(8, 28 );
       printf( "%.4lu", Isr);
       set_cursor_pos(9, 28 );
       printf( "%.2X", Sid);
       set_cursor_pos(10, 28 );
       printf( "%.4lu", P38Ports[0].ModemStatusRegister); 
       set_cursor_pos(11, 28 );
       printf( "%.4lu",P38Ports[0].ReceiveToutCtr); 

       if( P38Ports[1].PortState)
       {
           _asm    cli
           P38Ports[1].rcvbfrDrvr     = P38Ports[1].rcvbfrStart;
           P38Ports[1].rcvbfrPC       = P38Ports[1].rcvbfrStart;
           Error1 = P38Ports[1].Error1;
           Error2 = P38Ports[1].Error2;
           Sid = P38Ports[1].SidAccumlative;
           P38Ports[1].SidAccumlative = 0;
           P38Ports[1].Error1 = P38Ports[1].Error2 = 0;
           Isr =  P38Ports[1].ISREntry;
           _asm    sti

           set_cursor_pos(7, 55 );
           printf( "%.2X  %.2x", Error1, Error2);
           set_cursor_pos(8, 55 );
           printf( "%.4lu", Isr);
           set_cursor_pos(9, 55 );
           printf( "%.2X", Sid);
           set_cursor_pos(10, 55 );
           printf( "%.4lu", P38Ports[1].ModemStatusRegister); 
           set_cursor_pos(11, 55 );
           printf( "%.4lu",P38Ports[1].ReceiveToutCtr); 


       }






   } while ( quit != 1 );

   quit = 0;
}    


void   DisplayConfiguration(void)

{

   BYTE i;
   unsigned char Row;

   quit = 0;
   cls();

   set_cursor_pos(0, 0 );
   printf("\tPort Configuration Display   [Press any key to get Main menu..]");

   set_cursor_pos( 2, 25 );
   printf("P O R T   1");

   set_cursor_pos( 2, 50 );
   printf("P O R T   2");
   
   Row = 4;
   set_cursor_pos( Row++, 1 );
   printf("Enhanced address:");

   set_cursor_pos( Row++, 1 );
   printf("Memory Map address:");

   set_cursor_pos( Row++, 1 );
   printf("DMA Channel:");

   set_cursor_pos( Row++, 1 );
   printf("Receive Mode:");

   set_cursor_pos( Row++, 1 );
   printf("Transmit Mode:");

   set_cursor_pos( Row++, 1 );
   printf("BaudRate:");

   set_cursor_pos( Row++, 1 );
   printf("DataBits:");

   set_cursor_pos( Row++, 1 );
   printf("Stopbits:");

   set_cursor_pos( Row++, 1 );
   printf("Parity:");

   set_cursor_pos( Row++, 1 );
   printf("Flow Control byte 1:");

   set_cursor_pos( Row++, 1 );
   printf("Flow Control byte 1:");

   set_cursor_pos( Row++, 1 );
   printf("Receive Trigger:");

   set_cursor_pos( Row++, 1 );
   printf("IRQ:");

   set_cursor_pos( Row++, 1 );
   printf("Receive Move Size:");

   set_cursor_pos( Row++, 1 );
   printf("Transmit Move Size:");

   set_cursor_pos( Row++, 1 );
   printf("Uart Prescaler:");

   for (i = 0; i < 2; i++)
   {
       if( P38Ports[i].PortState)
       {
           DisplayConfigData((WORD) i);
       }
       else
       {
          set_cursor_pos( 4, (unsigned char)(25*(i+1)));
          printf("Port NOT Active.");
       }
   }

   getch();                                                                          



}    



void   DisplayConfigData(WORD Port)

{

   char    String[81];
   unsigned char Column, Row;

   if (!Port)
       Column = 25;
   else
       Column = 50;
  
   Row = 4;
   set_cursor_pos( Row++, Column );
   printf("%.4x", P38Ports[Port].EnhancedAddress);

   set_cursor_pos( Row++, Column );
   printf("%.4x", P38Ports[Port].MemoryMapAddress);

   set_cursor_pos( Row++, Column );
   printf("%.2x", P38Ports[Port].dmaChannel);

   set_cursor_pos( Row++, Column );
   switch( P38Ports[Port].RxMode)
   {
       case CHAR_RX:
           strcpy(String,"Programmed IO");
           break;    

       case DMA_RX:
           strcpy(String,"DMA");
           break;    

       case MEMORY_MAPPED:
           strcpy(String,"Memory Map");
           break;    
   }
   printf("%s", String);

   set_cursor_pos( Row++, Column );
   switch( P38Ports[Port].TxMode)
   {
       case CHAR_TX:
           strcpy(String,"Programmed IO");
           break;    

       case DMA_TX:
           strcpy(String,"DMA");
           break;    

       case MEMORY_MAPPED:
           strcpy(String,"Memory Map");
           break;    
   }
   printf("%s", String);

   set_cursor_pos( Row++, Column );
   ltoa((LONG) BaudRateTable[P38Ports[Port].bitRate ], String, 10);
   printf("%s", String);

   set_cursor_pos( Row++, Column );
   printf("%x", DataBitsTable[P38Ports[Port].dataBits].Value);

   set_cursor_pos( Row++, Column );
   strcpy (String, StopBitsTable[P38Ports[Port].stopBits].String);
   printf("%s", String);

   set_cursor_pos( Row++, Column );
   strcpy (String, ParityTable[P38Ports[Port].parityMode].String);
   printf("%s", String);

   set_cursor_pos( Row++, Column );
   printf("%x", P38Ports[Port].flowCtrlEsp1);

   set_cursor_pos( Row++, Column );
   printf("%x", P38Ports[Port].flowCtrlEsp2);

   set_cursor_pos( Row++, Column );
   printf("%x", P38Ports[Port].rxTrigger);

   set_cursor_pos( Row++, Column );
   strcpy (String, P38Ports[Port].IRQValues->CfgIRQ);
   printf("%s", String);

   set_cursor_pos( Row++, Column );
   if (P38Ports[Port].ReceiveMoveType == BYTE_IO)
       strcpy (String,"Byte");
   else
       strcpy (String,"Word");
   printf("%s", String);

   set_cursor_pos( Row++, Column );
   if (P38Ports[Port].TransmitMoveType == BYTE_IO)
       strcpy (String,"Byte");
   else
       strcpy (String,"Word");
   printf("%s", String);

   set_cursor_pos( Row++, Column );
   switch (P38Ports[Port].UartPrescaler)
   {
       case 0:
           strcpy (String,"1.8432");
           break;
       case 1:
           strcpy (String,"3.6864");
           break;
       case 2:
           strcpy (String,"7.3728");
           break;
       case 3:
           strcpy (String,"14.7456");
           break;
   }           
   printf("%s", String);




}



/********************************** ESP *******************************/


 
